/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.core.instancer;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Stack;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;

import org.eclipse.core.resources.IProject;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.NativeJavaObject;
import org.mozilla.javascript.NativeObject;
import org.mozilla.javascript.ScriptRuntime;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.WrapFactory;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.ITreeStorage;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.AbstractTreeStorage;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.core.ITreeStateListener;
import com.unitesk.requality.nodetypes.VirtualNode;

public class TreeInstancer extends AbstractTreeStorage {
    private TreeDB srcTree;
    private Map<String, Attribute> initalAttribute = new HashMap<String, Attribute>();
    private Map<UUID, Map<String,Attribute>> changedAttribute = new HashMap<UUID, Map<String,Attribute>>();
    private Map<UUID, Map<String,Attribute>> generatedAttribute = new HashMap<UUID, Map<String,Attribute>>();
    private Map<UUID,UUID> cloneParent = new ConcurrentHashMap<UUID, UUID>();
    private Map<UUID,UUID> parentReplace = new HashMap<UUID, UUID>();
    
    private static Map<String,Object> attributesToObjects(Collection<Attribute> collection){
    	Map<String,Object> vals = new HashMap<String, Object>();
    	for(Attribute a:collection)
    		vals.put(a.getKey(), a.getValue());
    	return vals;
    }
    
    public TreeInstancer(TreeDB db){
    	this(db,Collections.EMPTY_MAP);
    }
    
    public TreeInstancer(final TreeDB db,Map<String, Object> map) {
        this.initalAttribute = new HashMap<String, Attribute>();
        for(String s:map.keySet())
       		initalAttribute.put(s, new Attribute(db.getRootNode(), AttributeType.STRING, s, map.get(s).toString()));
        this.srcTree = db;
        loadChanges();
    }

    private ITreeStorage getStorage(){
    	TreeDB db = srcTree;
    	if(db.isVirtual())
    		db = VirtualNode.getBaseTree(db);
    	return db.getStorage();
    }
    
    @Override
    public UUID getRootNode() {
        //if(!nodePredicates.containsKey(getStorage().getRootNode())) {
          //  nodePredicates.put(getStorage().getRootNode(), getPredicate(getStorage().getRootNode()));
        //}
        return getStorage().getRootNode();
    }

    @Override
    public UUID getParent(UUID uuid) {
    	if(parentReplace.containsKey(uuid))
        	return parentReplace.get(uuid);
        return getStorage().getParent(uuid);
    }
    
    public String getErrors(){
    	UUID root = getRootNode();
    	Map<String,Object> attr = getAttributesMap(root);
    	if(attr!=null)
    	{
    		Object r = attr.get(TreeNode.ATTR_ERROR);
    		if(r != null)
    		return (String)r;
    	}
    	return "";
    }

    @Override
    public Collection<UUID> getChildren(UUID uuid) {
    	UUID baseuid = uuid;
    	//todo handle clones
    	boolean childClone = hasCloneParent(uuid);
    	UUID uid2 = uuid;
        Collection<UUID> result = new HashSet<UUID>();
        if(getStorage().canLoad(uuid) && !getNodeType(uuid).equals(VirtualNode.TYPE_NAME))
        	for(UUID uid : getStorage().getChildren(uuid))
        		result.add(childClone?cloneChild(uid,uuid,getNodeId(uid)):uid);
        for(UUID uid:parentReplace.keySet())
        	if(parentReplace.get(uid).equals(uuid))
        		result.add(uid);
       	uid2 = cloneParent.get(uid2);
       	if(uid2!=null && !VirtualNode.TYPE_NAME.equals(getNodeType(uid2)))
       		for(UUID uid:getChildren(uid2))
       			result.add(childClone?cloneChild(uid,uuid,getNodeId(uid)):uid);
        return result;
    }
    
    private void addToErrorAttribute(UUID id, String value){
    	if(!changedAttribute.containsKey(id))
    		changedAttribute.put(id, new HashMap<String, Attribute>());
    	Map<String,Attribute> map = changedAttribute.get(id);
    	String val = "";
    	if(map.containsKey(TreeNode.ATTR_ERROR))
    		val = (String)map.get(TreeNode.ATTR_ERROR).getValue()+"\n";
    	map.put(TreeNode.ATTR_ERROR, new Attribute(id, AttributeType.STRING,TreeNode.ATTR_ERROR,val+value));
    	changedAttribute.put(id, map);

//    	Map<String,Object> map2 = getAttributesMap(id);
//    	String s = value;
//    	if(map2.containsKey(TreeNode.ATTR_ERROR)){
//    		s = (String) map2.get(TreeNode.ATTR_ERROR) + "\n" + s;
//    	}
//    	map2.put(TreeNode.ATTR_ERROR, s);
//    	changedAttribute.put(id, map2);
    }
    
    private void registerError(UUID id, String error){
    	UUID root = getRootNode();
    	String nid = getQualifyingId(id);
    	addToErrorAttribute(id, error);
    	addToErrorAttribute(root, "Node '" +nid+"' has errors: "+error);
    }
    public UUID cloneChild(UUID uid, UUID newp, String string){
    	if (string == null)
    		string = "01";
    		//TODO temporal fix
    	UUID newid = UUID.randomUUID();
    	boolean find = false;
    	for(UUID pid : cloneParent.keySet())
    		if(cloneParent.get(pid).equals(uid))
   				if(parentReplace.containsKey(pid)&&parentReplace.get(pid).equals(newp)){
   					newid = pid;
   					find = true;
   				}
    	cloneParent.put(newid, uid);
    	parentReplace.put(newid, newp);

    	if(!find){
    		TreeDB db = srcTree;
    		if(!db.isVirtual())
    			db = VirtualNode.getVirtualTreeDB(db);
    		//Object oid = srcTree.getNode(uid).getId();
    		createNode(newp, newid, getNodeType(uid), string);
    		//TreeNode newNode = db.getNode(newid);
    		//if(canLoad(newNode.getParentUUId(true)))
    			//db.updateNodeChildren(newNode.getParent(true));
    	}
    	return newid;
    }
    public UUID cloneUUID(UUID uid, UUID masteruid, String id){
    	if (id == null)
    		id = "01";
    		//TODO what to do
    	UUID newid = UUID.randomUUID();
    	boolean find = false;
    	for(UUID pid : cloneParent.keySet())
    		if(cloneParent.get(pid).equals(uid))
   				if(parentReplace.get(pid)!=null && parentReplace.get(pid).equals(masteruid)&&(!changedAttribute.containsKey(pid)||
   						!changedAttribute.get(pid).containsKey("__cloneid")||changedAttribute.get(pid).get("__cloneid").getValue().equals(id))){
   					newid = pid;
   					find = true;
   				}
    	cloneParent.put(newid, uid);
    	parentReplace.put(newid, masteruid);
     	putAttribute(newid, new Attribute(newid, AttributeType.STRING, "_id", id));
    	putAttribute(newid, new Attribute(newid, AttributeType.STRING, "__cloneid", id));
    	if(!find){
    		TreeDB db = srcTree;
    		if(!db.isVirtual())
    			db = VirtualNode.getVirtualTreeDB(db);
    		createNode(masteruid, newid, getNodeType(uid), id);
    		TreeNode newNode = db.getNode(newid);
    		//if(canLoad(newNode.getParentUUId(true)))
    			//db.updateNodeChildren(newNode.getParent(true));
    	}
    	return newid;
    	
    }

    private String getQualifyingId(UUID id) {
		String s = getNodeId(id);
		UUID par = getParent(id);
		if(par!=null){
			String qid = getQualifyingId(par);
			if(qid.equals("/"))
				qid = "";
			s= qid +"/"+s;
		}
		return s;
	}

    public String evaluatePredicateString(UUID id, String predicate) {
    	if(predicate == null)
    		predicate = getPredicate(id);
        if(predicate==null || predicate.length()==0) return "";
        UUID sid = id;
    	while(hasCloneParent(id))
    		id = cloneParent.get(id);
        String contextResult = "";
        try {
            Context context = Context.enter();
            WrapFactory wrapFactory = new WrapFactory();
            context.setWrapFactory(wrapFactory);
            Scriptable scope = createScope(context, getAttributesMap(sid));
            //invalid return
            Object res = context.evaluateString( scope, predicate, "", 1, null );
            if(res instanceof NativeJavaObject)
            	contextResult = ((NativeJavaObject)res).unwrap().toString();
            else
            	contextResult = res.toString();
        }
        catch(Exception e){
        	return e.toString();
        } finally {
            Context.exit();
        }
        return contextResult;
    }
    
    public String checkPredicate(UUID id) {
    	return checkPredicate(id, null);
    }
    private Map<String, Object> attrsToValues(Map<String,Attribute>attrs){
    	Map<String,Object> vals = new HashMap<String, Object>();
    	for(String s : attrs.keySet())
    		vals.put(s, attrs.get(s).getValue());
    	return vals;
    }

	public String checkPredicate(Map<String,Attribute> attrs, String predicate) {
		if(predicate==null || predicate.equals(""))
			return "";
        String result = "1";
        result = evaluatePredicateWithErrors(predicate, attrsToValues(attrs));
        return result;
    }
	public String checkPredicate(UUID id, String predicate) {
        if(predicate == null)
    		predicate = getPredicate(id);
        String result = "1";
        if(predicate == null || predicate.equals(""))
        	return result;
        TreeDB db = getTreeDB();
        if(!db.isVirtual())
        	db = VirtualNode.getVirtualTreeDB(db);
        TreeNode node = db.getNode(id);
        result = node.evaluatePredicateString(node, predicate);
        return result;
    }

    public String getPredicate(UUID id) {
    	if(changedAttribute.containsKey(id)&&changedAttribute.get(id).containsKey(TreeNode.ATTR_PREDICATE)) {
            return (String) changedAttribute.get(id).get(TreeNode.ATTR_PREDICATE).getValue();
        }
        TreeDB db = srcTree;
        if(!db.isVirtual())
        	db = VirtualNode.getVirtualTreeDB(db);
        try{
        	TreeNode node = db.getNode(id);
        	if(node != null)
        		return node.getPredicate();
        }
        catch(Exception e){
        }
        return null;
        
    }
    
    private String concatPredicate(String full, String newPart) {
        if(full.length()==0) return newPart;
        if(newPart.length()==0)  return full;
            
        if(full.equals(newPart)) return full;
        return "("+full+") && ("+newPart+")";
    }
    
    private Map<String, Object> getAttributesMap(UUID id) {
    	return attributesToObjects(getAttributes(id));
    }

    private boolean evaluatePredicate(String predicate, Map<String, Object> map) {
    	String s = evaluatePredicateWithErrors(predicate, map);
    	return s.charAt(0)=='1';
    }
    private String evaluatePredicateWithErrors(String predicate, Map<String, Object> map) {
        if(predicate==null || predicate.length()==0) return "1";
        String error = "";
        Boolean contextResult=true;
        try {
            Context context = Context.enter();
            WrapFactory wrapFactory = new WrapFactory();
            context.setWrapFactory(wrapFactory);
            Scriptable scope = createScope(context, map);
            Object o = context.evaluateString( scope, predicate, "", 1, null );
            if(o instanceof Boolean)
            	contextResult = (Boolean)o;
            else{
            	if(o instanceof NativeJavaObject){
            		o = ((NativeJavaObject)o).unwrap();
            		if(o instanceof Boolean)
            			contextResult = (Boolean)o;
            		else
            			error+="'"+predicate+"' returns not boolean value";
            	}else
            		error+="'"+predicate+"' returns undefined value";
            }
        } catch(Exception e){
        	error+="'"+predicate+"' causes an error: "+e.getMessage();
        } finally {
            Context.exit();
        }
        return (contextResult?"1":"0")+error;
    }
    
    private Scriptable createScope(Context context, Map<String, Object> attributes) {
        Scriptable topScope = ScriptRuntime.initStandardObjects(context, null, true);
        Scriptable scope = new NativeObject();
        scope.setParentScope( null );
        scope.setPrototype( topScope );

        for(String attrName: attributes.keySet() ) {
            Object value = attributes.get(attrName);
        	if(!value.getClass().isArray())
        		value = value.toString();
            Object obj = context.getWrapFactory().wrap( context, scope, value, value.getClass() );
            scope.put( attrName, scope, obj );
        }

        return scope;
    }

    @Override
    public void createNode(UUID parentUUId, UUID uuid, String id,
            Collection<Attribute> attrs) {
        //throw new UnsupportedOperationException();
    }
    @Override
    public void remove(UUID node) {
//    	nodePredicates.put(node, "false");
    	putAttribute(node, new  Attribute(node, AttributeType.STRING, TreeNode.ATTR_PREDICATE, "false"));
    	putAttribute(node, new  Attribute(node, AttributeType.STRING, VirtualNode.ATTR_HIDDEN_STATUS, "true"));
    	getDeltaTarget().deleted(node, true);
    }
    public void realRemove(UUID node) {
    	realRemove(node,false);
    }
    public void realRemove(UUID node,boolean started) {
    	//TODO support it
    		Set<UUID>washere = new HashSet<UUID>();
    		ArrayList<UUID> its = new ArrayList<UUID>();
    		int i = 0;
    		its.add(node);
    		while(i<its.size()){
    			UUID uid = its.get(i);
    			washere.add(uid);
    			for(UUID id:getClones(uid))
    				if(!washere.contains(id))
    					its.add(id);
    			for(UUID id:getRepChildren(uid))
    				if(!washere.contains(id))
    					its.add(id);
    			i++;
    		}
    		for(int j=its.size()-1;j>=0;j--)
    		{
    			UUID uid = its.get(j);
    			cloneParent.remove(uid);
        		parentReplace.remove(uid);
        			
        		changedAttribute.remove(uid);
        		generatedAttribute.remove(uid);
        		getDeltaTarget().deleted(uid, true);
        		/*try{
        			remove(node);
        		}
        		catch(RuntimeException re){
        		}*/
    		}
    		
    		return;
        //throw new UnsupportedOperationException();
    }

    @Override
    public String getNodeId(UUID uuid) {
    	if(changedAttribute.containsKey(uuid) && changedAttribute.get(uuid).containsKey("_id"))
    		return (String)changedAttribute.get(uuid).get("_id").getValue();
    	if(hasCloneParent(uuid))
    		return getNodeId(cloneParent.get(uuid));
        return getStorage().getNodeId(uuid);
    }

    @Override
    public void changeNodeId(final UUID node, final String id) {
    	if(!changedAttribute.containsKey(node))
    		changedAttribute.put(node, new HashMap<String, Attribute>());
    	changedAttribute.get(node).put("_id", new Attribute(node, AttributeType.STRING,"_id", id));
    }

    public boolean hasCloneParent(UUID uid){
    	return cloneParent.containsKey(uid)&&!uid.equals(cloneParent.get(uid));
    }
    public Map<String,Attribute> getChangedAttributes(UUID uuid) {
    	if(changedAttribute.containsKey(uuid))
    		return changedAttribute.get(uuid);
    	return null;
    }
    @Override
    public Collection<Attribute> getAttributes(UUID uuid) {
    	if(!changedAttribute.containsKey(uuid)&&srcTree.getStorage().canLoad(uuid))
    		return srcTree.getStorage().getAttributes(uuid);
    	Map <String, Attribute> map = new HashMap<String, Attribute>();
    	UUID uuid2 = uuid;
    	Stack<UUID> pars = new Stack<UUID>();
    	pars.add(uuid2);
    	while(hasCloneParent(uuid2)){
    	    uuid2 = cloneParent.get(uuid2);
    	    if(hasCloneParent(uuid2))
    	    	pars.add(uuid2);
   	    }
    	if(getStorage().canLoad(uuid2)){
    		Collection<Attribute> attrs = getStorage().getAttributes(uuid2);
    		if(attrs!=null)
    			for(Attribute a:attrs)
    				if(a == null)
    				{
    				}else
    				map.put(a.getKey(), a);
    	}
    	while(!pars.isEmpty()){
    		UUID uid = pars.pop();
    		if(generatedAttribute.containsKey(uid))
        		for(Attribute a:generatedAttribute.get(uid).values())
        			map.put(a.getKey(), a);
    		//todo changed
    		if(changedAttribute.containsKey(uid))
        		for(Attribute a:changedAttribute.get(uid).values())
        			map.put(a.getKey(), a);
    	}
    	for(String s:initalAttribute.keySet())
    		if(!map.containsKey(s))
    			map.put(s, initalAttribute.get(s));
    	return map.values();
    }
    private void hideNode(UUID uid){
    	putAttribute(uid, new  Attribute(uid, AttributeType.STRING, TreeNode.ATTR_PREDICATE, "false"));
    	putAttribute(uid, new  Attribute(uid, AttributeType.STRING, VirtualNode.ATTR_HIDDEN_STATUS, "true"));
    }
    @Override
    public void saveAttributes(UUID node, Collection<Attribute> attributes) {
    	Map<String,Attribute> attrs = new HashMap<String,Attribute>();
    	if(cloneParent.containsKey(node))
    		for(Attribute a:getAttributes(cloneParent.get(node)))
    			attrs.put(a.getKey(), a);
    	saveAttributes(node, attributes, attrs);
    }
    private boolean equals(Object object, Object object2){
    	if(object==null && object2==null)
    		return true;
    	else
    		if(object!=null){
    			if(object instanceof List && object2 instanceof List)
    			{
    				List l1 = (List)object;
    				List l2 = (List)object2;
    				if(l1.size()!=l2.size())
    					return false;
    				for(int i =0;i<l1.size();i++)
    					if(!equals(l1.get(i),l2.get(i)))
    						return false;
    				return true;
    			}
    			else
    				return object.equals(object2);
    		}
    		else
    			return object2.equals(object);
    	
    }
    public void saveAttributes(UUID node, Collection<Attribute> attributes, Map<String,Attribute> parentAttribute) {
    	Map<String,Attribute>map = new HashMap<String, Attribute>(),genmap = new HashMap<String, Attribute>();
    	List<String> toupdate = new ArrayList<String>();
    	boolean changed = false;
    	for(Attribute a:attributes)
    	{
    		boolean skip = false;
    		Attribute b = null;
    		if(parentAttribute != null)
    			if(parentAttribute.containsKey(a.getKey())){
    				b = parentAttribute.get(a.getKey());
    				if(equals(b.getKey(),a.getKey())&&
    						(!a.getKey().startsWith("__")&&(b.getOrigin().equals(a.getOrigin())&&
    								(!b.getOrigin().equals(node)||equals(b.getRawValue(),a.getRawValue()))))){
    					skip = true;
    					if(!equals(b.getValue(),a.getValue()))
    						changed = true;
    				}
    			}
    		if(!skip){
    			changed = true;
    			if(!a.isGenerated()){
    				map.put(a.getKey(), a);
    			}else
    				genmap.put(a.getKey(), a);
    		}
    	}
    	if(changed){
    		changedAttribute.put(node, map);
    		generatedAttribute.put(node, genmap);
    		getDeltaTarget().changed(node, true);
    	}
    }
    private TreeDB getTreeDB(){
    	return VirtualNode.getVirtualTreeDB(srcTree);
    }
    public void setHidden(UUID uid, boolean state){
    	if(state){
    		putAttribute(uid, new  Attribute(uid, AttributeType.STRING, TreeNode.ATTR_PREDICATE, "false"));
    		putAttribute(uid, new  Attribute(uid, AttributeType.STRING, VirtualNode.ATTR_HIDDEN_STATUS, "true"));
    	}else
    		if(changedAttribute.containsKey(uid)&&changedAttribute.get(uid).containsKey(TreeNode.ATTR_PREDICATE)){
    			changedAttribute.get(uid).remove(VirtualNode.ATTR_HIDDEN_STATUS);
    			changedAttribute.get(uid).remove(VirtualNode.ATTR_PREDICATE);
    		}
    	
    }
    public boolean isHidden(UUID uid){
    	return changedAttribute.containsKey(uid)&&changedAttribute.get(uid).containsKey(TreeNode.ATTR_PREDICATE)&&changedAttribute.get(uid).get(TreeNode.ATTR_PREDICATE).getValue().equals("false");
    }
    public void saveChanges() {
    	if(readOnly)
    		return;
    	boolean changes = false;
    	if(srcTree.isVirtual())
    		srcTree=VirtualNode.getBaseTree(srcTree);
    	TreeNode root = null;
    	try{
    		root = srcTree == null? null :srcTree.getRootNode();
    	}
    	catch(Exception e){
    		//save unsucceed
    	}
    	//remove old info
    	Set<String> keys;
    	if(root==null || root.getAttributeKeys()==null)
    		keys = new HashSet<String>();
    	else
    		keys = new HashSet<String>(root.getAttributeKeys());
    	for(String s:keys)
    		if(s.startsWith("@")){
    			root.removeAttribute(s);
    			changes = true;
    		}
    	Set<UUID> tolook = new HashSet<UUID>(cloneParent.keySet());
    	//list of ids
    	for(UUID uid:tolook){
    		List<String>saved = new ArrayList<String>();
    		StringBuilder sb = new StringBuilder();
    		Map<String,Attribute> ca = changedAttribute.get(uid);
    		if(ca!=null)
    			for(String s:ca.keySet()){
    				Attribute a = ca.get(s);
    				root.rawPutAttribute(new Attribute(a.getOrigin(),a.getRawType(),"@"+uid.toString()+"|"+a.getKey(),a.getRawValue()));
    				changes = true;
    			}
    		if(cloneParent.containsKey(uid)||parentReplace.containsKey(uid)){
    			saved.add(cloneParent.get(uid).toString());
   				saved.add(parentReplace.get(uid)!=null?parentReplace.get(uid).toString():null);
    			root.rawPutAttribute(new Attribute(root, AttributeType.LIST,"@"+uid.toString(),saved));
    			changes = true;
    		}
    	}
    	if(root != null && root.getTreeDB().getStorage().canLoad(root.getUUId()) && changes)
    		root.rawSaveAttributes();
	}
    private void loadChanges(){
    	TreeNode root = srcTree.getRootNode();
    	generatedAttribute.clear();
    	cloneParent.clear();
    	changedAttribute.clear();
    	//TODO test-related stuff
    	if(root == null || root.getAttributeKeys()==null)
    		return;
		for(String s:root.getAttributeKeys()){
			if(s.startsWith("@")){
				Attribute a = root.getAttribute(s);
				if(s.indexOf("|")==-1){
					try{
						UUID uid = UUID.fromString(s.substring(1,s.length()));
						List<String> setts = (List<String>) a.getValue();
						if(setts.get(1)!=null&&!setts.get(1).equals(""))
							cloneParent.put(uid, UUID.fromString(setts.get(1)));
						if(setts.size()>2&&setts.get(2)!=null&&!setts.get(2).equals(""))
							parentReplace.put(uid, UUID.fromString(setts.get(2)));
					}catch(Exception e){
					}
				}else{
					UUID uid = UUID.fromString(s.substring(1,s.indexOf("|")));
					String name = s.substring(s.indexOf("|")+1);
					Attribute b = new Attribute(uid,a.getRawType(),name,a.getValue(),a.getRawValue(),a.getGenerator());
					if(!changedAttribute.containsKey(uid))
						changedAttribute.put(uid, new HashMap<String, Attribute>());
					changedAttribute.get(uid).put(name, b);
				}
			}
		}
    }

	@Override
    public void moveNodeTo(UUID node, UUID parent) {
    	UUID oldparent = getParent(node);
    	if(parent!=null)
    		parentReplace.put(node, parent);
    	else
    		parentReplace.remove(node);
    	getDeltaTarget().moved(node, oldparent, parent, true);
		//throw new UnsupportedOperationException();
    }

	@Override
	public String getNodeType(UUID uuid) {
		try{
			UUID uid = uuid;
			while(hasCloneParent(uid))
				uid = getCloneParent(uid);
			String s = getStorage().getNodeType(uid);
			if(s == null)
				return super.getNodeType(uuid);
			return s;
		}
		catch(Exception e){
			return null;
		}
	};
    @Override
    public boolean canLoad(UUID uuid) {
    	return (changedAttribute.containsKey(uuid)||cloneParent.containsKey(uuid)||parentReplace.containsKey(uuid))&&getNodeType(uuid)!=null||getStorage().canLoad(uuid);
    //	while(hasCloneParent(uuid))
    	//	uuid = cloneParent.get(uuid);
    	//if(!srcTree.getStorage().canLoad(uuid)){
//    		return changedAttribute.containsKey(uuid);
    	//}
   		//return true;
    }

    @Override
    public String getNodeResourcePath(UUID uuId) {
    	while(hasCloneParent(uuId))
    		uuId = cloneParent.get(uuId);
        return getStorage().getNodeResourcePath(uuId);
    }

    @Override
    public InputStream getResourceContents(UUID uuid, String resName) {
    	while(hasCloneParent(uuid))
    		uuid = cloneParent.get(uuid);
    	return getStorage().getResourceContents(uuid, resName);
    }

    @Override
    public void setResourceContents(UUID uuid, String resName, InputStream in, Object data) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean isResourceExists(UUID uuid, String resName) {
    	while(hasCloneParent(uuid))
    		uuid = cloneParent.get(uuid);
        return getStorage().isResourceExists(uuid, resName);
    }

    @Override
    public void deleteResource(UUID uuid, String resName) {
        // TODO: instancer resource support  
        throw new UnsupportedOperationException();
    }

    @Override
    public String[] listResources(UUID uuid) {
		while(hasCloneParent(uuid))
    		uuid = cloneParent.get(uuid);
		if(getStorage().canLoad(uuid))
			return getStorage().listResources(uuid);
		else//TODO look into
			return new String[0];
    }

    @Override
    public void createStorage(String name) {
        // TODO Auto-generated method stub
    }

    @Override
    public void deleteStorage() {
        // TODO Implement me!
        //throw new UnsupportedOperationException("not implemented");
    }

    @Override
    public String getName() {
        return ":"+srcTree.getName();
    }
    public List<UUID> getClones(UUID uuId) {
    	return getClones(uuId, null);
    }
	public List<UUID> getClones(UUID uuId, UUID uuid2) {
		List<UUID> uids = new ArrayList<UUID>();
		for(UUID id1:cloneParent.keySet())
			if(canLoad(id1) && cloneParent.get(id1).equals(uuId) && (uuid2==null || parentReplace.get(id1).equals(uuid2))){
				uids.add(id1);
			}
		return uids;
	}
	public UUID getCloneParent(UUID uid) {
		return cloneParent.get(uid);
	}
	//looks like cheat
	public void putAttribute(UUID uid, Attribute a){
		if(!a.isGenerated()){
			if(!changedAttribute.containsKey(uid))
				changedAttribute.put(uid, new ConcurrentHashMap<String, Attribute>());
			changedAttribute.get(uid).put(a.getKey(), a);
		}else{
			if(!generatedAttribute.containsKey(uid))
				generatedAttribute.put(uid, new ConcurrentHashMap<String, Attribute>());
			generatedAttribute.get(uid).put(a.getKey(), a);
		}
		
	}

	public List<UUID> getRepChildren(UUID uuId) {
		List<UUID> uids = new ArrayList<UUID>();
		Set<UUID> iter = new HashSet<UUID>(parentReplace.keySet());
		for(UUID id1:iter)
			if(id1!=null && parentReplace.containsKey(id1) && parentReplace.get(id1).equals(uuId)){
				if(canLoad(id1))
					uids.add(id1);
				else
					realRemove(id1);//TODO already deleted node
			}
		return uids;
	}
	private boolean readOnly = false;
	public void setReadOnly(boolean b) {
		readOnly = b;
	}

}

