package com.unitesk.requality.eclipse;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.ITreeNodeFilter;
import com.unitesk.requality.core.ITreeNodesFilter;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.eclipse.views.actions.NodeProcessingAction;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.Requirement.TransferStatus;

public class TranserredStatusSubMenu extends org.eclipse.ui.actions.CompoundContributionItem {
	public TranserredStatusSubMenu() {
	}
	
	@Override
	public boolean isEnabled() {
		List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
		if(nodes == null)
			return false;
		return someElementsTranferedFilter.select(nodes).size()>0;
	};
	@Override
	public boolean isVisible() {
		return isEnabled();
	};
	
    public static ITreeNodesFilter someElementsTranferedFilter = new ITreeNodesFilter() {
		@Override
		public List<TreeNode> select(List<TreeNode> nodes) {
			List<TreeNode> nd = new LinkedList<TreeNode>();
	    	for(TreeNode node:nodes)
	    		if(node.getAttribute(Requirement.ATTR_TRANSF_STATUS)!=null)
	    			nd.add(node);
	    	return nd;
		}
	};
    private ITreeNodeFilter elementTranferedFilter = new ITreeNodeFilter() {
		@Override
		public boolean select(TreeNode node) {
			return node.getAttribute(Requirement.ATTR_TRANSF_STATUS)!=null;
		}
    };
    	private ITreeNodeFilter getTSChangeAction(final Requirement.TransferStatus minstatus){
			return new ITreeNodeFilter() {
				@Override
				public boolean select(TreeNode node) {
					if(minstatus == null)
						node.removeAttribute(Requirement.ATTR_TRANSF_STATUS);
					else
						((Requirement)node).setTransferProblem(minstatus);
					return true;
				}
			};
    	}
		@Override
		protected IContributionItem[] getContributionItems() {
			List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
    		TransferStatus maxstatus = TransferStatus.FAILED,minstStatus = TransferStatus.COMPLETE;
    		boolean hasnoattr = false,hasattr = false;
    		if(nodes == null)
    			return new IContributionItem[0];
    		for(TreeNode node:nodes)
    		{
    			Attribute a = node.getAttribute(Requirement.ATTR_TRANSF_STATUS);
    			if(a==null){
    				maxstatus = TransferStatus.COMPLETE;
    				hasnoattr = true;
    				continue;
    			}
    			TransferStatus ts = Requirement.TransferStatus.valueOf((String)a.getValue());
    			if(ts.isBetterThan(maxstatus))
    				maxstatus = ts;
    			if(minstStatus.isBetterThan(ts))
    				minstStatus = ts;
    		}
    		if(!hasnoattr && maxstatus!=minstStatus)
    			maxstatus = Requirement.TransferStatus.values()[maxstatus.ordinal()-1];
			List<ContributionItem> items = new LinkedList<ContributionItem>();
			for(TransferStatus ts:Requirement.TransferStatus.values())
				if(ts.isBetterThan(maxstatus))
					items.add(new ActionContributionItem(new NodeProcessingAction("Set to "+ts.toString().toLowerCase(),elementTranferedFilter,someElementsTranferedFilter,getTSChangeAction(ts))));
			if(someElementsTranferedFilter.select(nodes).size()>0)
				items.add(new ActionContributionItem(new NodeProcessingAction("Remove",elementTranferedFilter,someElementsTranferedFilter,new ITreeNodeFilter() {
					@Override
					public boolean select(TreeNode node) {
						node.removeAttribute(Requirement.ATTR_TRANSF_STATUS);
						node.saveAttributes();
						return true;
					}
				})));
			return items.toArray(new IContributionItem[0]);
		}
		
}
