package com.unitesk.requality.eclipse.handlers;

import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;

import com.unitesk.requality.core.ITreeLogic.PastingIntoLeafException;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.VirtualNode;
import com.unitesk.requality.tools.RequalityCLI;

public class CreateInstancerHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        if(nodes == null || (nodes.size() > 1))
            return null;
        TreeNode req = nodes.get(0);
        String testPurposeId = RequalityCLI.getNextId(req, VirtualNode.TYPE_NAME, "");
        try {
            if(!req.getTreeDB().getTreeLogic().canBeChildOfExc(VirtualNode.TYPE_NAME, req)){
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Create Element",
                        "Element of type '" + splitWords(VirtualNode.TYPE_NAME)
                        + "' cannot be created in " + req.getUserFriendlyName());
                
                return null;
            }
        } catch (PastingIntoLeafException e) {
            MessageDialog.openError(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Create Node",
                    "Can't add Instancer to "+splitWords(req.getType())+" with non-Test Purpose child");
            return null;
        }
		if(req.getTreeDB().startTransaction("Create Instancer",true)){
			VirtualNode tc = (VirtualNode) req.createChild(testPurposeId,
					VirtualNode.TYPE_NAME);
			tc.saveAttributes();
			req.getTreeDB().commit();
			Activator.getDefault().setActiveNode(tc, VirtualNode.TYPE_NAME);
		}
        return null;
    }

    @Override
    public boolean isEnabled() {
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        if (nodes == null || (nodes.size() > 1)) {
            return false;
        }
        if (nodes.get(0).getTreeDB().isVirtual())
            return false;
        if(nodes.size() > 1)
            return false;
        if (!nodes.get(0).getTreeDB().getTreeLogic().
                canBeCreatedIn(VirtualNode.TYPE_NAME, nodes.get(0)))
                return false;
        return true;
    }
    private String splitWords(String s) {
        if (s == null)
            return null;
        for (int i = 1; i < s.length(); i++) //s.length() is variable
            if (s.charAt(i - 1) >= 'a' && s.charAt(i - 1) <= 'z' &&
                s.charAt(i) >= 'A' && s.charAt(i) <= 'Z') {
                s = s.substring(0, i) + " " + s.substring(i);
            }
        return s;
    }
}
