/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.handlers;

import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialogWithToggle;
import org.eclipse.swt.SWT;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.eclipse.views.RequalityPreferences;

public abstract class OpenEditorHandler extends AbstractHandler {
	public abstract String getEditorId();
	
	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
        if(RequalityPlugin.getBrowserType() == SWT.NONE){
        	if(Activator.getDefault().getPreferenceStore().getBoolean(RequalityPreferences.OPEN_NO_ENGINES_DIALOGS)){
        		MessageDialogWithToggle mdt = MessageDialogWithToggle.openInformation(Activator.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
					"Browser engines are not available",
					"There are no available browser engines. Please install XulRunner or WebKit. \nFollow instructions from http://requality.org/en/general.en.html#browser_integration. \nFurther work can lead to unexpected results.",
					"Do not show again", false, Activator.getDefault().getPreferenceStore(), RequalityPreferences.OPEN_NO_ENGINES_DIALOGS);
        		if(mdt.getToggleState())
        			Activator.getDefault().getPreferenceStore().setValue(RequalityPreferences.OPEN_NO_ENGINES_DIALOGS, false);
            	if(Activator.getDefault().getPreferenceStore().getBoolean(RequalityPreferences.NOT_OPEN_EDITORS))
            		return null;
        	}
        }
        return openOperation(event);
	}
	
	public Object openOperation(ExecutionEvent event) throws ExecutionException {
	    List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
	    for(TreeNode node : nodes){
	        NodeEditorInput input = new NodeEditorInput(node, getEditorId());
	        try {
	            PlatformUI.getWorkbench().getActiveWorkbenchWindow()
	                    .getActivePage()
	                    .openEditor(input, getEditorId(), false);
	        } catch (PartInitException e) {
	            e.printStackTrace();
	        }
	    }
		return null;
	}
	@Override
	public boolean isEnabled() {
	    if(RequalityPlugin.getBrowserType() == SWT.NONE)
        	if(Activator.getDefault().getPreferenceStore().getBoolean(RequalityPreferences.NOT_OPEN_EDITORS))
        		return false;
	    return true;
    }

}