package com.unitesk.requality.eclipse.handlers.copypaste;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.ITreeLogic.PastingIntoLeafException;
import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.CoreUtils;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.TreeLogic;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.Comment;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TestPurpose;
import com.unitesk.requality.tools.DeepFirstTreeWalker;
import com.unitesk.requality.tools.Index;
import com.unitesk.requality.tools.RequalityCLI;

public class PasteIntoHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        List<TreeNode> parent = RequalityCNF.getSelectedNodes();
        if(parent == null || parent.size() != 1)
        	return null;
        return pasteNodes(parent.get(0), true);
    }
    private boolean find(List<NodeDesc> nodes, String id){
    	for(NodeDesc node : nodes)
    		if(node.getId().equals(id))
    			return true;
    	return false;
    }
    private String getNewName(NodeDesc node, List<NodeDesc> sourceNodes, TreeNode parent){
    	String oldId = node.getId();
        String newId = oldId + "_copy";
        int copyCount = 1;
        while(parent.findChild(newId+copyCount) != null || find(sourceNodes, newId+copyCount))
            copyCount++;
        newId += copyCount;
        return newId;
    }
    public Object pasteNodes(TreeNode parent, boolean copyTree){
    	return pasteNodes(parent,copyTree,false);
    }
    public Object pasteNodes(TreeNode parent, boolean copyTree, boolean skipTransactionAndDialogs){
        List<NodeDesc> sourceNodes = null;
        if (!CopyPasteHelper.getInstance().isEmpty()) {
            TreeLogic l = parent.getTreeDB().getTreeLogic();
            sourceNodes = CopyPasteHelper.getInstance().getNodes();
            List<NodeDesc>toCopy = new ArrayList<NodeDesc>();
            TreeNode lastnode = null;
            for(NodeDesc node:sourceNodes)
            	if (!l.canBeChildOf( node.getType(), parent)
                    && l.canBeChildOf( node.getType(), parent.getParent())
                    && l.isSiblinkPasteAvalible(node.getType())&&
                    node.getType().equals(parent.getType()))
            	{ 
            		lastnode = parent;
            		parent = parent.getParent();
            	}
            if(lastnode == null && parent.sizeChildren() > 0)
                lastnode = parent.getSortedChildren()[parent.sizeChildren()-1];
            String toRename = "";
            String generalsstring = sourceNodes.size()>1?"s":"";
            int cnt = 0;
            SortedMap<String, Integer> cntMap = new TreeMap<String, Integer>();
            for(NodeDesc node : sourceNodes){
            	if(!cntMap.containsKey(node.getType()))
                	cntMap.put(node.getType(), 0);
                cntMap.put(node.getType(), cntMap.get(node.getType())+1);
            }
            for(NodeDesc node : sourceNodes){
                try {
                    if(l.canBeCreatedInExc(node.getType(), parent)){
                    	TreeNode old = parent.findChild(node.getId());
                        if(old != null && !l.isAutoRenamable(node.getType())){
                            toRename += old.getId() + ", ";
                            cnt++;
                        }
                        toCopy.add(node);
                    }else{
                    	if(!skipTransactionAndDialogs){
                    		MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Paste Element"+generalsstring,
                                "Element "+node.getId()+" cannot be copied to " + parent.getUserFriendlyName());
                    		return null;
                    	}else
                    		return "Element "+node.getId()+" cannot be copied to " + parent.getUserFriendlyName();
                    }
                } catch (PastingIntoLeafException e){
                    if(node != null){
                        if(node.getType().equals(TestPurpose.TYPE_NAME)){
                        	if(!skipTransactionAndDialogs){
                        		MessageDialog.openError(PlatformUI.getWorkbench()
                        				.getActiveWorkbenchWindow().getShell(), "Paste Element"+generalsstring,
                        				"Can't paste Test Purpose"+(cntMap.get(node.getType())>1?"s":"")+" in "+parent.getTypeLabel()+" with non-Test Purpose child");
                        		return null;
                        	}else
                        		return "Can't paste Test Purpose"+(cntMap.get(node.getType())>1?"s":"")+" in "+parent.getTypeLabel()+" with non-Test Purpose child";
                        }
                        if(!skipTransactionAndDialogs){
                        	MessageDialog.openError(PlatformUI.getWorkbench()
                        			.getActiveWorkbenchWindow().getShell(), "Paste Element"+generalsstring,
                        			"Can't paste "+  CoreUtils.splitWords(node.getType())+(!node.getType().endsWith("s")&&cntMap.get(node.getType())>1?"s":"")+" in "+parent.getTypeLabel()+" with Test Purpose child");
                        }else
                        	return "Can't paste "+CoreUtils.splitWords(node.getType())+(!node.getType().endsWith("s")&&cntMap.get(node.getType())>1?"s":"")+" in "+parent.getTypeLabel()+" with Test Purpose child";
                    }
                    return null;
                }
            }
            Activator a = Activator.getDefault();
            Shell shell = null;
            if(a!=null)
            	Activator.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell();
            String parentName = parent.getUserFriendlyName();
            if(parentName.equals(""))
                parentName = parent.getQualifiedId().substring(0,parent.getQualifiedId().indexOf("/"));
            String purposes = "",adds = cnt>1?"s":"";
            byte purpcnt = 0;
            if(!toRename.equals("")){
            	purposes+="\n- Element"+adds+" with id"+adds+" " + toRename.substring(0, toRename.length()-2) 
                        + " already exist"+(adds.equals("")?"s":"")+" in "
                        + parent.getUserFriendlyName() + ".";
            	purpcnt++;
            }
            boolean askrename = true;
            for(int i = 0;i<toCopy.size() && askrename;i++){
            	if(l.isAutoRenamable(toCopy.get(i).getType()))
            		continue;
                for(int j = i+1;j<toCopy.size();j++)
                    if(toCopy.get(i).getId().equals(toCopy.get(j).getId()))
                    {
                        if(l.isAutoRenamable(toCopy.get(i).getType()))
                            continue;
                        purposes += "\n- Selection contains elements with same ids.";
                        purpcnt++;
                        cnt+=2;
                        askrename = false;
                        break;
                    }
            }
            adds = cnt>1?"s":"";
            if(a==null)
            	skipTransactionAndDialogs = true;
            if(!purposes.equals(""))
            	if(!skipTransactionAndDialogs &&
            			!MessageDialog
                        .openConfirm(shell, "Paste Element"+generalsstring,
                                "Copied element"+(generalsstring.equals("")?" has":generalsstring+" have")+" id conflict"+(purpcnt>1?"s":"")+":\n" + purposes + "\n" 
                                        + "\nConflicting element"+adds+" will be renamed. If you still want to proceed click OK, otherwise click Cancel."))
                        {
                            return Status.CANCEL_STATUS;
                        }
           	if(!skipTransactionAndDialogs)
           		if(!parent.getTreeDB().startTransaction("Paste Element"+generalsstring,true))
           			return Status.CANCEL_STATUS;
            boolean flaglast = false;
            if (parent.sizeChildren() > 0 && lastnode == parent.getSortedChildren()[parent.sizeChildren()-1])
                flaglast = true;
            for(int i = 0; i < toCopy.size(); i++){
            	NodeDesc nodeDesc = toCopy.get(i);
            	String newId = null;
                if(!l.isChildIdValid(parent, nodeDesc.getId()))
                {
                    if(l.isAutoRenamable(nodeDesc.getType())){
                    	if(nodeDesc.getType().equals(Comment.TYPE_NAME))
                    		newId = RequalityCLI.getNextId(parent, nodeDesc.getType(), Comment.TYPE_NAME+" ");
                    	else
                    		newId = RequalityCLI.getNextId(parent, nodeDesc.getType(), null);
                   }else{
                   		newId = getNewName(nodeDesc, toCopy, parent);
                    }
                }
                String oldid = nodeDesc.getId();
                if(newId != null)
                    nodeDesc.setId(newId);
                if(!parent.getTreeDB().getTreeLogic().canBeCreatedIn(nodeDesc.getType(),
                        nodeDesc.getId(), parent))
                {
                	if(!skipTransactionAndDialogs){
                		parent.getTreeDB().rollback();
                		MessageDialog
                            .openError(shell, "Paste Element"+generalsstring,
                                    "Selected elements cannot be pasted together.");
                		return Status.CANCEL_STATUS;
                	}else
                		return "Selected elements cannot be pasted together.";
                }
                if(nodeDesc.getAttributes().get("_index") != null)
                	nodeDesc.setAttribute(new Attribute(nodeDesc.getUUId(), AttributeType.STRING, "_index", Index.getNextIndex(nodeDesc.getType(), parent, lastnode, nodeDesc.getType().equals("TestStep")).toString()));
                
                TreeNode newnode = nodeDesc.createNode(parent.getTreeDB(), parent.getUUId(),
                        true);
                // reset location names doesn't affect treedb
               	resetLocationNames(newnode);
               	if(flaglast && parent.sizeChildren() > 0){
                    lastnode = parent.getSortedChildren()[parent.sizeChildren()-1];
                }else{
                    lastnode = newnode;
                }
                nodeDesc.setId(oldid);
            }
            if(!skipTransactionAndDialogs)
            	parent.getTreeDB().commit();
       }
       return null;
    }

    private void resetLocationNames(TreeNode root) {
        for (TreeNode node : new DeepFirstTreeWalker(root)) {
            if (node.getType().equals(Requirement.TYPE_NAME)) {
            	List<String> toremove = new LinkedList<String>();  
                for(String s: ((Requirement) node).getLocationQIds()){
                	try{
                		((Location)(node.getTreeDB().getNode(s))).resetLabel();
                	}
                	catch (Exception e) {
                		 toremove.add(s);
					}
                }
                for(String s : toremove)
                	((Requirement) node).removeLocation(s);
                if(!toremove.isEmpty())
                	node.saveAttributes();
            }
        }
    }

    @Override
    public boolean isEnabled() {
        List<TreeNode>nodes = RequalityCNF.getSelectedNodes();
        if(nodes == null || nodes.size() > 1)
            return false;
        TreeNode target = nodes.get(0);
        if(CopyPasteHelper.getInstance().isEmpty())
            return false;
        List<NodeDesc> sourceNodes = CopyPasteHelper.getInstance().getNodes();
        TreeLogic l = target.getTreeDB().getTreeLogic();
        for(NodeDesc node : sourceNodes)
        	if (!l.canBeChildOf(node.getType(), target) 
                && l.canBeChildOf( node.getType(), target.getParent())
                && node.getType().equals(target.getType()))
                    return true;
        for(NodeDesc node:sourceNodes)
            if(!l.canBeCreatedIn(node.getType(), target))
                return false;
        return true;
    }
}