/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.handlers.document;

import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.tools.RequalityCLI;

public class CreateDocFolderHandler extends AbstractHandler {
	@Override
	public boolean isEnabled(){
		List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        if(nodes == null || (nodes.size() > 1))
            return false;
		return true;
	}
    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        if(nodes == null || (nodes.size() > 1))
            return null;
        TreeNode node = nodes.get(0);
        final DocFolder parent;
        if (node instanceof Document) {
            parent = (DocFolder) node.getParent();
        } else if (node instanceof DocFolder) {
            parent = (DocFolder) node;
        } else {
            throw new UnsupportedOperationException("Wrong selection type");
        }

        InputDialog dlg = new InputDialog(PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getShell(), "New Document Folder",
                "Enter folder name", RequalityCLI.getNextId(parent, DocFolder.TYPE_NAME, "new_folder_"),
                new IInputValidator() {
                    @Override
                    public String isValid(String newText) {
                        newText = newText.trim();
                        if (newText.length() == 0) {
                            return "enter folder name";
                        }
                        if (newText.lastIndexOf("/") != -1) {
                            return "folder name can't contains '/'";
                        }
                        if (parent.findChild(newText) != null) {
                            return "child with same id is exist";
                        }
                        if (!parent.getTreeDB().getTreeLogic().canBeCreatedIn(DocFolder.TYPE_NAME, parent))
                            return "This folder cannot contain " + newText;
                        return null;
                    }
                });

        if (dlg.open() == Dialog.OK) {
            if(parent.getTreeDB().startTransaction("Create Document Folder",true)){
            	TreeNode folder = parent.createChild(dlg.getValue(), DocFolder.TYPE_NAME);
            	parent.getTreeDB().commit();
            	Activator.getDefault().setActiveNode(folder, DocFolder.TYPE_NAME);
            }
        }
        return null;
    }
}
