package com.unitesk.requality.eclipse.handlers.location;

import java.util.List;
import java.util.Map;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.commands.IElementUpdater;
import org.eclipse.ui.menus.UIElement;
import org.eclipse.ui.part.IPage;
import org.eclipse.ui.views.properties.PropertySheet;
import org.eclipse.ui.views.properties.tabbed.ISection;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class RemoveLocationFromRequirementHandler extends AbstractHandler implements IElementUpdater{

    private ILocationTab getTab() {
        IWorkbenchPart part = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActivePart();
        if (part instanceof PropertySheet) {
            IPage page = ((PropertySheet) part).getCurrentPage();
            if (page instanceof TabbedPropertySheetPage) {
            	if (((TabbedPropertySheetPage) page).getCurrentTab() == null)
            		return null;
                ISection[] sections = ((TabbedPropertySheetPage) page).getCurrentTab().getSections();
                if (sections != null && sections.length == 1 && sections[0] instanceof ILocationTab) {
                    return (ILocationTab) sections[0];
                }
            }
        }
        return null;
    }
  //TODO
    @SuppressWarnings("rawtypes")
    @Override
    public void updateElement(UIElement element, Map parameters) {
    	ILocationTab tab = getTab();
        if (tab == null) {
            return;
        }
        Map<Location, List<Requirement>> map = tab.getLocationsAndRequirements();
    	element.setText("Delete Location"+((map!=null && (map.keySet().size()>1||(map.keySet().size()>0 && map.values().iterator().next().size() > 1)))?"s":""));
    }
    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        ILocationTab tab = getTab();
        if (tab == null) {
            return null;
        }
        Map<Location, List<Requirement>> map = tab.getLocationsAndRequirements();
        if (map == null || map.size() == 0) {
            return null;
        }
        //req == null - delete all links
    	String s = map.size() > 1 ? "s" : "";
        if (!MessageDialog.openConfirm(PlatformUI.getWorkbench()
                   .getActiveWorkbenchWindow().getShell(), "Delete Location"+s,
                   "Delete selected Location"+s+"?")) {
               return null;
        }
        Location l = map.keySet().iterator().next();
        if(l.getTreeDB().startTransaction("Delete Location"+s,true)){
        	for(Location lc:map.keySet()){
        		List<Requirement>reqs = map.get(lc);
        		if(reqs == null || reqs.size() == 0){
        			Requirement[]lreq = l.getRequirements();
        			for(Requirement rq : lreq)
        				reqs.add(rq);
        		}
        		for(Requirement r : reqs)
        			CommonRemoveLocation.RemoveLocation(lc, r, null, true);
        	}
        	l.getTreeDB().commit();
        	tab.refreshLocations();
        }
        return null;
    }

    @Override
    public boolean isEnabled() {
        ILocationTab tab = getTab();
        if (tab == null)
        	return false;
        Map<Location, List<Requirement>> map = tab.getLocationsAndRequirements();
        if(map == null || map.size() == 0) {
            return false;
        }
        return true;
    }
    

}
