/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.handlers.report;

import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.ITreeLogic.IdContainsWrongChars;
import com.unitesk.requality.core.ITreeLogic.IdEmpty;
import com.unitesk.requality.core.ITreeLogic.PastingIntoLeafException;
import com.unitesk.requality.core.ITreeLogic.SameIdExistsInChildren;
import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.CoreUtils;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.ReportSettings;
import com.unitesk.requality.nodetypes.ReportFolder;
import com.unitesk.requality.report.Template;
import com.unitesk.requality.tools.RequalityCLI;

public class CreateReportSettingsHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        if(nodes == null || (nodes.size() > 1))
            return null;
        TreeNode node = nodes.get(0);
        String id = RequalityCLI.getNextId(node, ReportSettings.TYPE_NAME, null);
        // TODO report undo
        if(node.getTreeDB().startTransaction("Create Report Data",true)){
        	try {
        		if(!node.getTreeDB().getTreeLogic().canBeCreatedInExc(ReportSettings.TYPE_NAME, id, node)){
        			MessageDialog.openError(PlatformUI.getWorkbench()
        					.getActiveWorkbenchWindow().getShell(), "Add Element",
        					"Element of type '" + CoreUtils.splitWords(ReportSettings.TYPE_NAME) 
        					+ "' cannot be created in " + node.getUserFriendlyName());
        			return null;
        		}
        	} catch (PastingIntoLeafException e) {
        		MessageDialog.openError(PlatformUI.getWorkbench()
        				.getActiveWorkbenchWindow().getShell(), "Add Element",
        				"Can't add "+CoreUtils.splitWords(ReportSettings.TYPE_NAME)+" to "+node.getTypeLabel()+" with "
        						+ "Test Purpose child");
        		return null;
        	} catch (SameIdExistsInChildren e) {
        		MessageDialog.openError(PlatformUI.getWorkbench()
        				.getActiveWorkbenchWindow().getShell(), "Add Element",
        				"Element with same id exists in " + 
        						node.getUserFriendlyName());
        		return null;
        	} catch (IdEmpty e) {
        		MessageDialog.openError(PlatformUI.getWorkbench()
        				.getActiveWorkbenchWindow().getShell(), "Add Element",
        				"Element id is empty");
        		return null;
        	} catch (IdContainsWrongChars e) {
        		MessageDialog.openError(PlatformUI.getWorkbench()
        				.getActiveWorkbenchWindow().getShell(), "Add Element",
        				"Element id contains wrong character - " 
        						+ e.getWrongCharacter());
        		return null;
        	}
        	ReportSettings newReportSettings = (ReportSettings) node.createChild(id,
        			ReportSettings.TYPE_NAME);
        	newReportSettings.putAttribute(new Attribute(newReportSettings, AttributeType.STRING, ReportSettings.ATTR_TEMPLATE, Template.DEFAULT_ID));
        	newReportSettings.saveAttributes();
        	node.getTreeDB().commit();
        	Activator.getDefault().setActiveNode(newReportSettings, ReportSettings.TYPE_NAME);
        }
        return null;
    }

    @Override
    public boolean isEnabled() {
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        if(nodes == null || (nodes.size() > 1))
            return false;
        TreeNode node = nodes.get(0);
        if (!node.getType().equals(ReportFolder.TYPE_NAME)) {
            return false;
        }
        return true;
    }
}
