/*
 * Copyright 2009-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a cop
import com.unitesk.requality.eclipse.views.RequalityContentProvider;
y of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.ui.dialogs;

import java.util.HashSet;
import java.util.List;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.model.WorkbenchLabelProvider;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.ITreeLogic.PastingIntoLeafException;
import com.unitesk.requality.core.CoreUtils;
import com.unitesk.requality.core.ITreeNodeFilter;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeLogic;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.core.DefaultTreeChangeListener;
import com.unitesk.requality.eclipse.ui.cnf.RequalitySorter;
import com.unitesk.requality.eclipse.views.RequalityContentProvider;
import com.unitesk.requality.nodetypes.TestPurpose;
import com.unitesk.requality.nodetypes.VirtualNode;
import com.unitesk.requality.tools.RequalityCLI;

abstract public class SelectNodeDialog extends TreeViewerDialog<TreeNode> {
    private HashSet<TreeNode> newNodes = new HashSet<TreeNode>();
    
    private final class NodeSelectionListener implements
            ISelectionChangedListener {
        @Override
        public void selectionChanged(SelectionChangedEvent event) {
            TreeSelection ts = (TreeSelection) event.getSelection();
            if (!isSelectedSame(selectedNode)) {
                if (ts.getFirstElement() instanceof TreeNode) {
                    select(ts.getFirstElement());
                } else if (ts.getFirstElement() instanceof TreeDB) {
                    TreeDB db = (TreeDB) ts.getFirstElement();
                    select(db.getNode(getRootQId()));
                }
                updateUI();
            }
        }
    }

    private ITreeChangeListener catalogChangeListener = new DefaultTreeChangeListener() {

        private boolean dirty = false;

        @Override
        public void changed(TreeNode node) {
            if (dirty) {
                return;
            }
            dirty = true;
            Shell shell = getShell();
            if (shell == null)
                return;

            Display display = shell.getDisplay();
            if (display.isDisposed())
                return;

            display.asyncExec(new Runnable() {
                @Override
                public void run() {
                    refresh();
                    dirty = false;
                }
            });
        }
    };


    private static final String TEXT_CREATE_REQUIREMENT = "Create New";

    private static int TREE_WIDTH = 250;
    private static int TREE_HEIGHT = 300;

    public String getTitle() {
        return "Element selection";
    }

    protected TreeDB db = null;

    private boolean canSelectRoot = false;

    protected String newNodeType = null;

    protected String childtype = null;
    public void setTypeOfCreatedChilden(String t){
        childtype = t;
    }
    private final SelectionListener createButtonListener = new SelectionListener() {
        @Override
        public void widgetSelected(SelectionEvent e) {
            TreeLogic l = selectedNode.getTreeDB().getTreeLogic();
            try {
                if((childtype != null && !l.canBeCreatedInExc(childtype, selectedNode))
                        || (childtype == null && !l.canBeCreatedInExc(selectedNode.getType(), selectedNode))){
                    MessageDialog.openError(PlatformUI.getWorkbench()
                            .getActiveWorkbenchWindow().getShell(), "Create Element",
                            "Element of type '" +childtype!=null?CoreUtils.splitWords(childtype):selectedNode.getTypeLabel() 
                            + "' cannot be created in " + selectedNode.getUserFriendlyName()+"("+selectedNode.getTypeLabel()+")");
                    return;
                }
            } catch (PastingIntoLeafException e1) {
                String type = (childtype!=null?childtype:selectedNode.getType());
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Create Element",
                        "Can't add "+ CoreUtils.splitWords(type) 
                        + " to "+selectedNode.getTypeLabel()+" with " + (type.equals(TestPurpose.TYPE_NAME)?"non-":"")
                        + "Test Purpose child");
                return;
            }
            GetNodeIdDialog dialog = new GetNodeIdDialog(
                    SelectNodeDialog.this.getShell(), selectedNode,
                    RequalityCLI.getNextId(selectedNode, newNodeType != null ? newNodeType : selectedNode.getType(), defaultIdPrefix()));
            if ((dialog.open() == GetNodeIdDialog.OK)
                    && (selectedNode.getTreeDB() != null)) {
                TreeNode newNode = null;
                if(selectedNode.getTreeDB().startTransaction("Create Element",true)){
                	if(childtype != null){
                		newNode = selectedNode.createChild(dialog.getResult().trim(),childtype);
                	}else{
                		newNode = selectedNode.createChild(dialog.getResult().trim());
                	}
                	saveAction(newNode);
                	selectedNode.saveAttributes();
                	
                	selectedNode.getTreeDB().commit();
                	
                	newNodes.add(newNode);
                	select(newNode);
                	// 	ReqTreeView.setSelected((TreeNode) selectedNode);
                }
            }
        }

        @Override
        public void widgetDefaultSelected(SelectionEvent e) {
            widgetDefaultSelected(e);
        }
    };

    protected String defaultIdPrefix(){ return null; } 
    
    protected void saveAction(TreeNode n){};

    @Override
    public int open() {
        db.addGUIListener(catalogChangeListener);
        return super.open();
    }

    @Override
    protected void buttonPressed(int buttonId) {
        super.buttonPressed(buttonId);
        db.removeGUIListener(catalogChangeListener);
    }

    public boolean canSelectNode() {
    	if(filter != null &&
        		!filter.select(selectedNode)){
        	return false;
        }
    	if(super.canSelectNode())
    		return true;
    	if (canSelectRoot) {
            return true; // we already select something
        }
        boolean result = !getRootQId().equals(selectedNode.getQualifiedId());
        return result;
    }

    public SelectNodeDialog(Shell shell, TreeDB treedb) {
        super(shell);
        this.db = treedb;
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Composite container = createClientContainer(parent, 1);
        header = new Composite(container, SWT.NONE);
        header.setLayout(new GridLayout(2, false));
        Label commentLabel = new Label(header, SWT.NONE);
        String labelString = "Select an element";
        if (canCreateNodes()) {
            labelString += " or ";
            createButton = new Button(header, SWT.PUSH);
            createButton.setText(TEXT_CREATE_REQUIREMENT);
            createButton.addSelectionListener(createButtonListener);
        }
        commentLabel.setText(labelString);
        commentLabel.setLayoutData(new GridData(GridData.BEGINNING,
                GridData.CENTER, true, true));
        nodeTree = new TreeViewer(container);
        nodeTree.getControl().setLayoutData(
                new GridData(TREE_WIDTH, TREE_HEIGHT));
        nodeTree.setLabelProvider(getLabelProvider());
        nodeTree.setSorter(new RequalitySorter());
        nodeTree.setAutoExpandLevel(2);
        nodeTree.setComparator(new RequalitySorter());
        if (db == null) {
            nodeTree.setContentProvider(getContentProvider());
            nodeTree.setInput(ResourcesPlugin.getWorkspace().getRoot());
        } else {
            nodeTree.setContentProvider(getContentProvider());
            nodeTree.setInput(db.getNode(getRootQId()));
        }

        nodeTree.addSelectionChangedListener(new NodeSelectionListener());

        nodeTree.getTree().addMouseListener(mouseList);
        nodeTree.getTree().addKeyListener(keyList);

        this.getShell().setText(getTitle());
        this.getShell().setImage(
                PlatformUI.getWorkbench().getSharedImages()
                        .getImage(ISharedImages.IMG_OBJ_ELEMENT));

        refresh();

        return container;
    }

    protected ILabelProvider getLabelProvider() {
        return WorkbenchLabelProvider
                .getDecoratingWorkbenchLabelProvider();
    }

    public boolean canCreateNodes() {
        return false;
    }

    protected ITreeContentProvider getContentProvider() {
        return new RequalityContentProvider(false);
    }

    protected Composite createClientContainer(Composite parent, int span) {
        Composite container = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.marginWidth = layout.marginHeight = 10;
        layout.verticalSpacing = 2;
        layout.numColumns = span;
        container.setLayout(layout);
        return container;
    }

    public void setCanSelectRoot(boolean canSelectRoot) {
        this.canSelectRoot = canSelectRoot;
    }


    public void select(Object obj) {
        if (obj != null) {
            selectedNode = (TreeNode) obj;
            if (nodeTree != null && !nodeTree.getControl().isDisposed()) {
            	if(createButton!=null)
            		createButton.setEnabled(!selectedNode.isVirtual()&&!VirtualNode.TYPE_NAME.equals(selectedNode.getType()));
                nodeTree.expandToLevel(selectedNode, 0);
                nodeTree.setSelection(new StructuredSelection(obj), true);
                updateUI();
            }
        }
    }

    private boolean isSelectedSame(TreeNode selectedReq) {
        IStructuredSelection selection = (IStructuredSelection) nodeTree
                .getSelection();
        if (selection.getFirstElement() == null)
            return false;
        if (!selection.getFirstElement().equals(selectedReq))
            return false;
        return true;
    }

    @Override
    protected void refresh() {
        if (nodeTree != null && !nodeTree.getControl().isDisposed()) {
            nodeTree.refresh();

            if (selectedNode != null) {
                // Because base can be reloaded
                selectedNode = selectedNode.getTreeDB().getNode(
                        selectedNode.getUUId());
                if (!isSelectedSame(selectedNode)) {
                    select(selectedNode);
                }
            }
        }
    }

    public boolean isNewNodeSelected() {
        return newNodes.contains(selectedNode);
    }

    private ITreeNodeFilter filter;
	public void setFilter(ITreeNodeFilter f) {
		filter = f;
	}
}
