/*
 * Copyright 2009-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.views.actions;

import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.swt.SWT;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.IHandlerService;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.eclipse.views.RequalityPreferences;

public class OpenAction extends Action {
    @Override
    public boolean isEnabled() {
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        if (nodes == null) {
            return false;
        }
        if(RequalityPlugin.getBrowserType() == SWT.NONE)
        	if(Activator.getDefault().getPreferenceStore().getBoolean(RequalityPreferences.NOT_OPEN_EDITORS))
        		return false;
        return true;
    }

    public void run() {
        try {
            runAction();
        } catch (PartInitException e) {
            e.printStackTrace();
        }
    }

    private void runAction() throws PartInitException {
        if (!isEnabled()) {
            return;
        }
        IHandlerService handlerService = (IHandlerService) PlatformUI
                .getWorkbench().getActiveWorkbenchWindow().getActivePage()
                .getActivePart().getSite().getService(IHandlerService.class);
        try {
            handlerService.executeCommand(
                    "com.unitesk.requality.commands.OpenNode", null);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
