/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.UIJob;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeGenerator;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.INodeChangeListener;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.TreeNodeComparator;
import com.unitesk.requality.core.transaction.TransactionStorage;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.ui.dialogs.EditAttrDialog;
import com.unitesk.requality.eclipse.views.atypeeditors.BoolValueEditor;
import com.unitesk.requality.eclipse.views.atypeeditors.FloatValueEditor;
import com.unitesk.requality.eclipse.views.atypeeditors.GeneratorValueEditor;
import com.unitesk.requality.eclipse.views.atypeeditors.IntValueEditor;
import com.unitesk.requality.eclipse.views.atypeeditors.LinkValueEditor;
import com.unitesk.requality.eclipse.views.atypeeditors.ListValueEditor;

public class AttributesPanel implements INodeChangeListener {
    private final class AttributeTableContentProvider implements
            IStructuredContentProvider {
        @Override
        public Object[] getElements(Object input) {
            if (input instanceof TreeNode) {
                TreeNode node = (TreeNode) input;
                ArrayList<Attribute> data = new ArrayList<Attribute>();
                for (String key : node.getAttributeKeys()) {
                    if (!key.startsWith("_")) {
                        data.add(node.getAttribute(key));
                    }
                }
                data.add(new Attribute(node,AttributeType.STRING,"",AttributeType.getDefaultValue(AttributeType.STRING)));
                return data.toArray(new Object[0]);
            }
            return null;
        }

        @Override
        public void dispose() {
        }

        @Override
        public void inputChanged(Viewer arg0, Object arg1, Object arg2) {
        }
    }

    private Composite panel;
    private static final String columnNames[] = new String[] { "Name", "Type", "Value"};
    private TreeNode node;
    private TableViewer attribTable;
    private boolean enabled = true;
    private void insertAttribute(){
        EditAttrDialog addAttrDialog = new EditAttrDialog(
                panel.getShell(), AttributeType.STRING, "", "", makeAttrMap());
        if (addAttrDialog.open() == EditAttrDialog.OK && node.getTreeDB().startTransaction("Update "+node.getType(),true)) {
            node.putAttribute(new Attribute(node, AttributeType.STRING,
                    addAttrDialog.getKey(), addAttrDialog.getValue()));
            attribTable.refresh();
            attribTable.getControl().setFocus();
            panel.notifyListeners(SWT.Modify, new Event());
            //node cannot be equal to null, otherwise it will throw
            //exception 
            node.putAttribute(new Attribute(node, AttributeType.STRING,
                    addAttrDialog.getKey(), addAttrDialog
                            .getValue()));
            node.saveAttributes();
            node.getTreeDB().commit();
        }
    }
    private void removeNode(String name,boolean startTransaction){
    	if(startTransaction)
    		//TODO may be message
    		if(!node.getTreeDB().startTransaction("Update "+node.getType(),true))
    			return;
        node.removeAttribute(name);
        node.saveAttributes();
        attribTable.refresh();
        attribTable.getControl().setFocus();
        panel.notifyListeners(SWT.Modify, new Event());
        if(startTransaction)
        	node.getTreeDB().commit();
    }
    private void removeSelection(){
    	TableItem[] items = attribTable.getTable().getSelection();
    	int cnt = 0;
    	for (int i = 0; i < items.length; i++) {
            Object attrName = ((Attribute) items[i].getData()).getKey();
            if(!attrName.equals(""))
            	cnt++;
    	}
    	if(cnt == 0)
    		return;
    	//TODO message on fail
    	if (node.getTreeDB().getStorage() instanceof TransactionStorage)
    		return;
        String mes = items.length <= 1 ? "Are you sure you want to remove selected attribute?"
                : "Are you sure you want to remove selected attributes?";

        if ((items.length > 0)
                && (MessageDialog.openQuestion(panel.getShell(),
                        "Attribute Editor", mes) == true)) {
        	if(node.getTreeDB().startTransaction("Update "+node.getType(),true)){
        		for (int i = 0; i < items.length; i++) {
        			Object attrName = ((Attribute) items[i].getData()).getKey();
        			if(attrName.equals(""))
        				continue;
        			removeNode(attrName.toString(),false);
        		}
        		node.getTreeDB().commit();
        	}
        }
    }
    private Listener removeAttrLink = new Listener() {
        @Override
        public void handleEvent(Event event) {
            if ((event.type == SWT.KeyDown && event.keyCode == SWT.DEL)
                    || (event.type == SWT.Selection)) {
            	removeSelection();
            }
        }
    };

    private Map<String, Object> makeAttrMap() {
        Map<String, Object> innerAttr = new HashMap<String, Object>();
        for (String key : node.getAttributeKeys()) {
            innerAttr.put(key, node.getAttributeValue(key).toString());
        }
        return innerAttr;
    }
    private Map<String,Integer> order = new HashMap<String,Integer>();
    private int compare(Attribute a, Attribute b){
    	if(!order.containsKey(a.getKey()))
    		order.put(a.getKey(), a.getKey().equals("")?Integer.MAX_VALUE:order.size());
    	if(!order.containsKey(b.getKey()))
    		order.put(b.getKey(), b.getKey().equals("")?Integer.MAX_VALUE:order.size());
    	return order.get(a.getKey()).compareTo(order.get(b.getKey()));
    }
    private ControlDecoration createDecorator(Text text) {
        ControlDecoration controlDecoration = new ControlDecoration(text,
                SWT.LEFT | SWT.TOP);
        FieldDecoration fieldDecoration = FieldDecorationRegistry.getDefault()
                .getFieldDecoration(FieldDecorationRegistry.DEC_ERROR);
        controlDecoration.setImage(fieldDecoration.getImage());
        return controlDecoration;
    }
    private Map<AttributeType, CellEditor> valueEditors;
	private GeneratorValueEditor generatorEditor;
    public AttributesPanel(Composite parent, TabbedPropertySheetWidgetFactory wf) {
        GridLayout layout = new GridLayout(2, false);
        layout.marginHeight = 0;
        layout.marginWidth = 0;
    	
    	panel = wf.createComposite(parent, SWT.NONE);
        panel.setLayout(layout);

        Label label = wf.createLabel(panel, "Attributes:", SWT.NONE);
        label.setLayoutData(new GridData());
        
        Composite buttons = wf.createComposite(panel, SWT.NONE);
        FillLayout buttonsLayout = new FillLayout();
        buttonsLayout.spacing = 3;
        buttons.setLayout(buttonsLayout);
        buttons.setLayoutData(new GridData());
        
        attribTable = new TableViewer(panel, SWT.H_SCROLL | SWT.V_SCROLL
                | SWT.BORDER | SWT.RESIZE |SWT.FULL_SELECTION );
        attribTable.getControl().addKeyListener(new KeyListener() {
			
			@Override
			public void keyReleased(KeyEvent e) {
				// TODO Auto-generated method stub
			}
			
			@Override
			public void keyPressed(KeyEvent e) {
				// TODO Auto-generated method stub
				//if(e.keyCode == SWT.INSERT)
					//insertAttribute();
				if(e.keyCode == SWT.DEL)
					removeSelection();
			}
		});
        attribTable.setSorter(new ViewerSorter() {
            private int getVal(String s) {
                int a = 0;
                for (int i = 0; i < s.length() && s.charAt(i) >= '0'
                        && s.charAt(i) <= '9'; i++)
                    a = a * 10 + s.charAt(i) - '0';
                return a;
            }

            public int compare(Viewer viewer, Object e1, Object e2) {
                if(e1 instanceof Attribute && e2 instanceof Attribute){
                	Attribute a1=(Attribute)e1,a2=(Attribute)e2;
                	return AttributesPanel.this.compare(a1,a2);
                	//return TreeNodeComparator.compareStrings(a1.getKey(), a2.getKey());
                }
                return super.compare(viewer, e1, e2);
            }
        });
        GridData gridData = new GridData(GridData.FILL_BOTH);
        gridData.horizontalSpan = 2;
        attribTable.getControl().setLayoutData(gridData);
        
        // Create columns
        int i = 0;
     // Editors
        final CellEditor[] editors = new CellEditor[columnNames.length];
        editors[1] = new TextCellEditor(attribTable.getTable(),SWT.FULL_SELECTION);
        editors[1].setValidator(new ICellEditorValidator() {
			@Override
			public String isValid(Object value) {
				String s = (String) value;
				if(node.getAttribute(s)!=null)
					return "Attribute with this name already exists";
				if(s.startsWith("_"))
					return "Attribute name cannot be started with '_' character";
				if(!Attribute.checkNameIsValid(s, false))
					return "Name contains illegal characters or starts with _ or $";
				return null;
			}
		});
        String[]types = new String[AttributeType.visibleValues().length];
        int j = 0;
        for(AttributeType t:AttributeType.visibleValues())
        	types[j++] = t.toString();
        ComboBoxCellEditor ed = new ComboBoxCellEditor(attribTable.getTable(), types,SWT.FULL_SELECTION);
        editors[2] = ed;
        valueEditors = new HashMap<AttributeType, CellEditor>();
        valueEditors.put(AttributeType.STRING, new TextCellEditor(attribTable.getTable(),SWT.FULL_SELECTION));
        valueEditors.put(AttributeType.BOOL, new BoolValueEditor(attribTable.getTable(),SWT.FULL_SELECTION));
        valueEditors.put(AttributeType.INT, new IntValueEditor(attribTable.getTable(),SWT.FULL_SELECTION));
        valueEditors.put(AttributeType.FLOAT, new FloatValueEditor(attribTable.getTable(),SWT.FULL_SELECTION));
        //valueEditors.put(AttributeType.RESOURCE, new ResourceValueEditor(attribTable.getTable(),SWT.FULL_SELECTION));
        //editors[3] = new TextCellEditor(attribTable.getTable(),SWT.FULL_SELECTION);
        //editors[4] = new TextCellEditor(attribTable.getTable(),SWT.FULL_SELECTION);
        TableViewerColumn clink =  new TableViewerColumn(attribTable,SWT.FULL_SELECTION);
        clink.getColumn().setResizable(false);
        clink.getColumn().setWidth(25);
        clink.setEditingSupport(new EditingSupport(attribTable) {
			@Override
			protected void setValue(Object element, Object value) {
			}
			
			@Override
			protected Object getValue(Object element) {
				return "";
			}
			
			@Override
			protected CellEditor getCellEditor(Object element) {
				return new TextCellEditor();
			}
			
			@Override
			protected boolean canEdit(Object element) {
				if(((Attribute)element).getKey().equals(""))
					return false;
				UUID uuid = ((Attribute)element).getOrigin();
				if(uuid == null || uuid == node.getUUId())
					return false;
				TreeNode nde = node.getTreeDB().getNode(uuid);
				final Map<String,List<TreeNode>> map = new HashMap<String, List<TreeNode>>();
				map.put(nde.getType(), new ArrayList<TreeNode>());
				map.get(nde.getType()).add(nde);
				Job j = new WorkspaceJob("Test") {
					
					@Override
					public IStatus runInWorkspace(IProgressMonitor monitor)
							throws CoreException {
						Activator.getDefault().setActiveNodes(map);
						return Status.OK_STATUS;
					}
				};
				j.schedule();
				//attribTable.remove(element);
				//removeNode(((Attribute)element).getKey(),true);
				return false;
			}
		});
    	i = 1;
    	attribTable.addDoubleClickListener(new IDoubleClickListener() {
			
			@Override
			public void doubleClick(DoubleClickEvent event) {
				updateSorting();
				panel.update();
				}
		});
    	for (String colName : columnNames) {
            TableViewerColumn column = new TableViewerColumn(attribTable,
                    SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
            column.getColumn().setText(colName);
            column.getColumn().setWidth(100);
            column.getColumn().setResizable(true);
            column.getColumn().setMoveable(true);
            final int iv = i;
            column.getColumn().addListener(SWT.Selection, new Listener() {
				@Override
				public void handleEvent(Event event) {
					updateSorting(sortingMethod == iv-1 ? sortingMethod + 10 : iv-1);
					attribTable.refresh(true);
				}
			});
        	column.setEditingSupport(new EditingSupport(column.getViewer()) {
				
				@Override
				protected void setValue(final Object element, Object value) {
					if(!getCellEditor(element).isDirty() || getCellEditor(element).getValidator()!=null && getCellEditor(element).getValidator().isValid(value)!=null || value == null){
						//TODO message
						return;
					}
					Attribute a = (Attribute)element;
					if(a.getKey().equals("")&&(iv!=1||(value.toString().trim()).equals("")))
						return;
					if(node.getTreeDB().startTransaction("Update "+node.getType(),true)){
						String key = a.getKey();
						Attribute old=null;
						AttributeType at = a.getRawType();
						if(a.getGenerator()!=null)
							at = a.getGenerator().getAttributeType();
						boolean changed = false;
						if(iv==1){
							if(!a.getKey().equals(""))
								node.removeAttribute(a.getKey());
							old=node.putAttribute(new Attribute(node.getUUId(), a.getRawType(), ((String)value).trim(),a.getValue(), a.getRawValue(),a.getGenerator()));
							key=((String)value).trim();
							changed = true;
						}
						if(iv==2 && (Integer)value > -1 && (Integer)value!=a.getType().ordinal()){
							old=node.putAttribute(new Attribute(node.getUUId(), AttributeType.visibleValues()[(Integer)value], a.getKey(), AttributeType.getDefaultValue(AttributeType.visibleValues()[(Integer)value])));
							changed = true;
						}
						if(iv==3){
							if(a.getType().equals(AttributeType.BOOL))
								value = (value.equals(1))?"true":"false";
							changed = true;
							old=node.putAttribute(new Attribute(node.getUUId(), at, a.getKey(), value));
						}
						if(iv==4)
							if(a.getRawType().equals(AttributeType.GENERATED)){
								old=node.putAttribute(new Attribute(node.getUUId(), at, a.getKey(), a.getValue(), a.getRawValue(),(AttributeGenerator)value));
								changed = true;
							}
						
						Attribute now = node.getAttribute(key);
						if(changed &&( old==null && now!=null || now == null && old!=null || now!=null && !now.equals(old)||now!=null && now.isGenerated())){
							if(a!=null&&!a.getKey().equals(""))
							{
								order.put(now.getKey(), order.get(a.getKey()));
								//order.remove(a.getKey());
							}else
								order.put(now.getKey(), order.size());
							node.saveAttributes();
							node.getTreeDB().commit();
						}else
							node.getTreeDB().rollback();
					}
			        attribTable.refresh();
			        attribTable.getControl().setFocus();
			        panel.notifyListeners(SWT.Modify, new Event());

				}
				
				@Override
				protected Object getValue(Object element) {
					if(element == null)
						return "";
					Attribute a = (Attribute)element;
					if(a.getKey().equals(""))
						return "";
					if(iv==1)
						return a.getKey();
					if(iv==2)
						return a.getType().ordinal();
					if(iv==3){
						if(a.getType().equals(AttributeType.BOOL))
							return a.getRawValue().toString().toLowerCase().equals("true")?1:0;
						return a.getRawValue();
					}
					return "";
				}
				
				@Override
				protected CellEditor getCellEditor(Object element) {
					Attribute a = (Attribute)element;
					if(iv == 3)
						return valueEditors.get(a.getType());
					return editors[iv];
				}
				
				@Override
				protected boolean canEdit(Object element) {
					if(!enabled)
						return false;
					return !((Attribute)element).getKey().equals("")&&iv<=3&&iv>0 ||iv==1;//||iv==2&&((Attribute)element).getRawType().equals(AttributeType.STRING);
				}
    		});
        	i++;
        }
        TableViewerColumn cgen =  new TableViewerColumn(attribTable,SWT.FULL_SELECTION);
        cgen.getColumn().setWidth(100);
        cgen.getColumn().setText("Generator");
        cgen.setEditingSupport(new EditingSupport(attribTable) {
			@Override
			protected void setValue(Object element, Object value) {
				if(value == null||!getCellEditor(element).isDirty())
					return;
				if(node.getTreeDB().startTransaction("Update "+node.getType(),true)){
					Attribute a = (Attribute) element;
					TreeNode master = node.getTreeDB().getNode((((AttributeGenerator) value)).getMasterUUId());
					Object v = ((AttributeGenerator) value).getValue(master);
					a = new Attribute(a.getOrigin(), AttributeType.GENERATED, a.getKey(), v,v ,((AttributeGenerator) value).copy());
					node.putAttribute(a);
					node.saveAttributes();
					node.getTreeDB().commit();
					attribTable.refresh();
					attribTable.getControl().setFocus();
					panel.notifyListeners(SWT.Modify, new Event());
				}
			}
			
			@Override
			protected Object getValue(Object element) {
				Attribute a = (Attribute) element;
				AttributeGenerator ag = a.getGenerator();
				if(ag!=null &&  a.getRawValue() instanceof Object[])
					ag.setRandValues((Object[]) a.getRawValue());
				return ag;
			}
			
			@Override
			protected CellEditor getCellEditor(Object element) {
				return generatorEditor;
			}
			
			@Override
			protected boolean canEdit(Object element) {
				if(((Attribute)element).getKey().equals(""))
					return false;
				
				//attribTable.remove(element);
				//removeNode(((Attribute)element).getKey(),true);
				return true;
			}
		});
    	
        TableViewerColumn cdel =  new TableViewerColumn(attribTable,SWT.FULL_SELECTION);
    	cdel.getColumn().setResizable(false);
    	cdel.getColumn().setWidth(25);
    	cdel.setEditingSupport(new EditingSupport(attribTable) {
			@Override
			protected void setValue(Object element, Object value) {

			}
			
			@Override
			protected Object getValue(Object element) {
				return "";
			}
			
			@Override
			protected CellEditor getCellEditor(Object element) {
				return new TextCellEditor();
			}
			
			@Override
			protected boolean canEdit(Object element) {
				if(((Attribute)element).getKey().equals("")||!enabled)
					return false;
				if(node.getTreeDB().startTransaction("Update "+node.getType(),true)){
					attribTable.remove(element);
					removeNode(((Attribute)element).getKey(),false);
					node.getTreeDB().commit();
				}
				return false;
			}
		});
    	
        attribTable.setColumnProperties(columnNames);
        attribTable.getTable().setHeaderVisible(true);
        attribTable.getTable().setLinesVisible(true);
        //attribTable.setCellEditors(editors);
        attribTable.setLabelProvider(new ITableLabelProvider() {
            @Override
            public void removeListener(ILabelProviderListener arg0) {
            }

            @Override
            public void addListener(ILabelProviderListener arg0) {
            }

            @Override
            public boolean isLabelProperty(Object arg0, String arg1) {
                return true;
            }

            @Override
            public void dispose() {
            }

            @Override
            public String getColumnText(Object element, int columnIndex) {
                Attribute attr = (Attribute) element;
                if(attr.getKey().equals(""))
                	return "";
                if (columnIndex == 1) {
                    return attr.getKey();
                }
                if (columnIndex == 2) {
                    return attr.getType().toString();
                }
                if (columnIndex == 3) {
                    if (attr == null || attr.getValue()==null) return "null";
                    Object val = attr.getValue();
                    if(attr.getType()==AttributeType.LIST)
            			val = new Object[]{attr.getListType(),val};
            		if(attr.getType() == AttributeType.REFERENCE)
            			return node.getLinkLabel(attr.getKey());
                    return Attribute.valueToString(attr.getType(), val);
                }
          /*      if (columnIndex == 3){
                	TreeNode par = node;
                	if(attr.getOrigin()!=node.getUUId() && attr.getOrigin() != null)
                	{
                		while(par != null && par.getUUId() != attr.getOrigin())
                			par = par.getParent(true);
                	}
                	if(par == null)
                		par = node;
                	AttributeGenerator rag = attr.getGenerator();
                	String type = "USER-DEFINED";
                	if(rag!=null)
                		type = rag.getAvailability().toString();
                	return par.getUserFriendlyName();
                }
                if (columnIndex == 4){
                	String type = "USER-DEFINED";
                	if(!attr.isGenerated())
                		return type;
                	TreeNode par = node;
                	if(attr.getOrigin()!=node.getUUId() && attr.getOrigin() != null)
                	{
                		while(par != null && par.getUUId() != attr.getOrigin())
                			par = par.getParent(true);
                	}
                	if(par == null)
                		par = node;
                	AttributeGenerator rag = attr.getGenerator();
                	if(rag!=null)
                		type = rag.getAvailability().toString();
                	return type;
                }*/
                if(columnIndex == 4)
                	return attr.getGenerator()==null?"":attr.getGenerator().toString();
                if(columnIndex == 0 || columnIndex == 5)
                	return "";
                return "n/a";
            }

            @Override
            public Image getColumnImage(Object element, int columnIndex) {
            	Attribute a = ((Attribute)element);
            	if(!a.getKey().equals("")){
            		if(columnIndex == 0 && !a.getOrigin().equals(node.getUUId()))
            			return PlatformUI.getWorkbench().getSharedImages()
            					.getImage(ISharedImages.IMG_TOOL_BACK);
            		if(columnIndex == 5 && enabled)
            			return PlatformUI.getWorkbench().getSharedImages()
            					.getImage(ISharedImages.IMG_ELCL_REMOVE);
            	}
            	
            	return null;
            }
        });
        attribTable.setContentProvider(new AttributeTableContentProvider());
   /*     attribTable.addDoubleClickListener(new IDoubleClickListener() {
            @Override
            public void doubleClick(DoubleClickEvent event) {
                Attribute attribute = (Attribute) ((IStructuredSelection) event
                        .getSelection()).getFirstElement();
                StructuredSelection s = (StructuredSelection) event.getSelection();
                if(attribute.isGenerated())
                	return;
                String savedOldName = (String) attribute.getKey();
                EditAttrDialog addAttrDialog = new EditAttrDialog(getControl()
                        .getShell(), attribute.getType() , (String) attribute.getKey(),
                        attribute.getValue(), makeAttrMap());
                if (addAttrDialog.open() == EditAttrDialog.OK) {
                    node.getTreeDB().startTransaction("Update "+node.getType());
                    if (!savedOldName.equals(addAttrDialog.getKey())) {
                        node.removeAttribute(savedOldName);
                    }
                    node.putAttribute(new Attribute(node, attribute.getType(),
                            addAttrDialog.getKey(), addAttrDialog.getValue()));
                    node.saveAttributes();
                    node.getTreeDB().commit();
                    attribTable.refresh();
                    attribTable.getControl().setFocus();
                    panel.notifyListeners(SWT.Modify, new Event());
                    attribTable.refresh();
                }
            }
        });*/
    }
    public void updatePanel(){
        attribTable.refresh();
        attribTable.getControl().setFocus();
        panel.notifyListeners(SWT.Modify, new Event());

    }
    public Control getControl() {
        return panel;
    }
    private int sortingMethod = 0;
    private void updateSorting(){
    	updateSorting(sortingMethod);
    }
    private Comparator<Attribute> attrSorter = new Comparator<Attribute>() {

		@Override
		public int compare(Attribute o1, Attribute o2) {
			int s = sortingMethod % 10;
			boolean s2 = sortingMethod / 10==1;
			switch (s) {
			case 0:
				return s2?o2.getKey().compareTo(o1.getKey()):o1.getKey().compareTo(o2.getKey());
			case 1:
				return s2?o2.getType().compareTo(o1.getType()):o1.getType().compareTo(o2.getType());
			case 2:
				return s2?o2.getValue().toString().compareTo(o1.getValue().toString()):o1.getValue().toString().compareTo(o2.getValue().toString());
			case 3:
				return s2?o2.getGenerator().toString().compareTo(o1.getGenerator().toString()):o1.getGenerator().toString().compareTo(o2.getGenerator().toString());
								
			default:
				break;
			}
			return 0;
		}
	};
    private void updateSorting(int sorting){
    	if(this.sortingMethod!=sorting)
    		order.clear();
    	this.sortingMethod = sorting;
    	if(this.node!=null){
    		Attribute[]s = node.getAttributes().values().toArray(new Attribute[0]);
    		Arrays.sort(s,attrSorter);
    		for(Attribute a:s)
    			order.put(a.getKey(), order.size());
    	}
    }
    public void setNode(TreeNode node) {
    	if(this.node!=null)
    		this.node.removeChangeListener(this);
    	boolean b = !node.equals(this.node); 
        this.node = node;
    	if(b){
    		order.clear();
    		updateSorting();
    	}
        node.addChangeListener(this);
        //TODO temp;
        valueEditors.put(AttributeType.REFERENCE, new LinkValueEditor(node, attribTable.getTable(),SWT.FULL_SELECTION));
        valueEditors.put(AttributeType.LIST, new ListValueEditor(node, attribTable.getTable(),SWT.FULL_SELECTION));
        generatorEditor = new GeneratorValueEditor(node, attribTable.getTable(), SWT.FULL_SELECTION);
        //valueEditors.put(AttributeType.GENERATED, );
        attribTable.setInput(node);
    }
//    private void setEnabledRec(boolean b, Control c){
    	
//    	if(c instanceof Composite)
  //  		for(Control cn:((Composite)c).getChildren())
    //			setEnabledRec(b, cn);;
  //  }
	public void setEnabled(boolean b) {
		//getControl().setEnabled(b);
		enabled = b;
		attribTable.refresh();
	}
	@Override
	public void attributeChange(final TreeNode source, String what) {
		new UIJob("refresh attribute panel") {
			@Override
			public IStatus runInUIThread(IProgressMonitor monitor) {
				if(attribTable.getControl().isDisposed())
					source.removeChangeListener(AttributesPanel.this);
				else
					attribTable.refresh();
				return Status.OK_STATUS;
			}
		}.schedule();;
	}
	@Override
	public void deleted(final TreeNode source) {
		new UIJob("refresh attribute panel") {
			@Override
			public IStatus runInUIThread(IProgressMonitor monitor) {
				if(attribTable.getControl().isDisposed())
					source.removeChangeListener(AttributesPanel.this);
				return Status.OK_STATUS;
			}
		}.schedule();
	}
	@Override
	public void moved(final TreeNode source, TreeNode oldParent, TreeNode newParent) {
		new UIJob("refresh attribute panel") {
			@Override
			public IStatus runInUIThread(IProgressMonitor monitor) {
				if(attribTable.getControl().isDisposed())
					source.removeChangeListener(AttributesPanel.this);
				return Status.OK_STATUS;
			}
		}.schedule();
	}
}
