/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.ITreeLogic.IdContainsWrongChars;
import com.unitesk.requality.core.ITreeLogic.IdEmpty;
import com.unitesk.requality.core.ITreeLogic.SameIdExistsInChildren;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class MainPropertySection extends TreeNodePropertiesTab {

    private AttributesPanel attribPane;
    private Text textId;
    private Text textName;
    private Label textNameLabel;
    ControlDecoration reqIdDecorator;
    private TreeNode req;
    protected boolean nameEnabled = true;
    public MainPropertySection() {
    }

    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        TabbedPropertySheetWidgetFactory wf = getWidgetFactory();
        
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));
        parent.setLayout(new GridLayout(2, false));
        
        wf.createLabel(parent, "Id:").setLayoutData(
                new GridData(GridData.HORIZONTAL_ALIGN_END)
        );
        textId = wf.createText(parent, "");
        textId.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        reqIdDecorator = createDecorator(textId);
        reqIdDecorator.hide();
        textId.addKeyListener(new KeyListener() {
			
			@Override
			public void keyReleased(KeyEvent e) {
				try {
	                if (req == null) {
	                    return;
	                }
	                
	                if (req.getTreeDB().getTreeLogic().isIdValidExc(req, textId.getText().trim())) {
	                    reqIdDecorator.hide();
	                } else {
	                    reqIdDecorator.setDescriptionText("Requirement Id is not correct and Requirement Id should not be empty");
	                    reqIdDecorator.show();
	                }
	            } catch (SameIdExistsInChildren e1) {
	                reqIdDecorator.setDescriptionText("Requirement with the same Id already exists");
	            } catch (IdEmpty e1) {
	                reqIdDecorator.setDescriptionText("Requirement Id is empty");
	            } catch (IdContainsWrongChars e1) {
	                reqIdDecorator.setDescriptionText("Requirement Id contains wrong character - " + e1.getWrongCharacter());
	            }
			}
			
			@Override
			public void keyPressed(KeyEvent e) {}
		});
        textId.addFocusListener(new FocusAdapter() {
        	public void focusLost(FocusEvent e) {
        		if(updating)
					return;
                if (!textId.getText().equals(textId.getText().trim())) {
                    textId.setText(textId.getText().trim());
                }
                if (req == null) {
                    return;
                }
                if (req.getId().equals(textId.getText())) {
                    return;
                }
                if (!req.getQualifiedId().equals(Requirement.getTypeRootQId())) {
                    String newId = textId.getText().trim();
                    if (req.getTreeDB().getTreeLogic().isIdValid(req, newId)&& req.getTreeDB().startTransaction("Change Requirement Id",true)) {
                        req.setId(newId.trim());
                        req.getTreeDB().commit();
                    } else {
                        textId.setText(req.getId());
                    }
                }
            }
        });
        textNameLabel = wf.createLabel(parent, "Name:");
        textNameLabel.setLayoutData(
                new GridData(GridData.HORIZONTAL_ALIGN_END)
        );
        textNameLabel.setVisible(nameEnabled);
        textName = wf.createText(parent, "");
        textName.setVisible(nameEnabled);
        textName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        textName.addFocusListener(new FocusAdapter() {
        	public void focusLost(FocusEvent e) {
        		if(updating)
					return;
        		if (!textName.getText().equals(textName.getText().trim())) {
                    textName.setText(textName.getText().trim());
                }
                if (req == null) {
                    return;
                }
                if (req.getName() == null && textName.getText().equals("") || req.getName() != null && req.getName().equals(textName.getText())) {
                    return;
                }
                if(req.getTreeDB().startTransaction("Update Requirement",true)){
                	req.setName(textName.getText());
                	req.saveAttributes();
                	req.getTreeDB().commit();
                }else
                	textName.setText(req.getName()!=null?req.getName():"");
            }
        });
        
        GridData gridData = new GridData(GridData.FILL_BOTH);
        gridData.horizontalSpan = 2;
        attribPane = new AttributesPanel(parent, wf);
        attribPane.getControl().setLayoutData(gridData);
    }

    private ControlDecoration createDecorator(Text text) {
        ControlDecoration controlDecoration = new ControlDecoration(text,
                SWT.LEFT | SWT.TOP);
        FieldDecoration fieldDecoration = FieldDecorationRegistry.getDefault()
                .getFieldDecoration(FieldDecorationRegistry.DEC_ERROR);
        controlDecoration.setImage(fieldDecoration.getImage());
        return controlDecoration;
    }

    @Override
    protected void setTreeNode(TreeNode node) {
    	nameEnabled = false;
        if (node.isDisposed()) {
            return;
        }
        req = node;
        if(node instanceof Location)
        	req = toRequirement(node);
        if (req == null || textId == null || textId.isDisposed() || req.isDisposed()) {
            return;
        }
        if (req.getParent() != null && !req.getParent().getQualifiedId().equals("/")) {
            textId.setEnabled(true);
        } else {
            textId.setEnabled(false);
        }
        if(req.getType().equals(Requirement.TYPE_NAME))
    		nameEnabled = true;
        if (!textId.getText().equals(req.getId())) { //against blinking
            textId.setText(req.getId());
        }
        textName.setVisible(nameEnabled);
        textNameLabel.setVisible(nameEnabled);
        if(!nameEnabled){
        	textName.setLayoutData(new GridData(0,0));
        	textNameLabel.setLayoutData(new GridData(0,0));
        }else
        {
        	textName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        	textNameLabel.setLayoutData(new GridData(GridData.BEGINNING));
        }
        textName.getParent().layout();
        textName.getParent().redraw();
        if(nameEnabled){
        	String name = req.getName() != null ? req.getName() : "";
        	if (!textName.getText().equals(name)) { //against blinking
        		textName.setText(name);
        	}
        }
        attribPane.setNode(req);
    }
}
