package com.unitesk.requality.eclipse.views.properties;

import org.eclipse.core.resources.IProject;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchPropertyPage;
import org.eclipse.ui.dialogs.PropertyPage;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.core.TreesTracker;

public class ProjectProperties extends PropertyPage implements
		IWorkbenchPropertyPage {
	private Combo sbllist;
    private TreeNode nodeToStore;
    private IProject p;
    private Text helpLabel;
    //text on top of preference page
    private final static String desciption_text = 
            "Sort method:";
    private final static String LOCATIONS_TEXT="Requirements are sorted by location position in document. "
    		+ "If there are several documents sorting depends on document names. "
    		+ "If requirement has several locations sorting depends only on the first location. "
    		+ "Requirements without locations are at the end of the list and are sorted by name.",
    		NAMES_TEXT="Requirements are sorted by name (first, in ascending order, then in alphabetical order). "
    				+ "In case of coincidence or lack of names requirements are sorted by id.";
    //information about link to manual
    public ProjectProperties() {
    }

    @Override
    protected Control createContents(Composite parent) {
     
        final Group container2 = new Group(parent, SWT.NONE);
        container2.setText("Sort options:");
        container2.setLayout(new GridLayout(2,false));
        container2.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        
        Label l = new Label(container2, SWT.NONE);
        l.setLayoutData(new GridData());
        l.setText(desciption_text);
        
        sbllist = new Combo(container2, SWT.NONE);
        sbllist.add("By Names");
        sbllist.add("By Locations");
        sbllist.select(getSortByLocationsState()?1:0);
        sbllist.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if(helpLabel!=null){
    				helpLabel.setText(sbllist.getSelectionIndex()>0?LOCATIONS_TEXT:NAMES_TEXT);
    				container2.layout();
				}
			}
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
        
        sbllist.setLayoutData(new GridData());

        helpLabel = new Text(container2, SWT.WRAP);
        
        GridData gd = new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1);
		gd.horizontalSpan = 2;
		gd.widthHint = 200;
		helpLabel.setLayoutData(gd);
		helpLabel.setBackground(container2.getBackground());
		helpLabel.setText(getSortByLocationsState()?LOCATIONS_TEXT:NAMES_TEXT);
		container2.layout();
		
        
        return container2;
    }
    @Override 
    public void setElement(org.eclipse.core.runtime.IAdaptable element) {
    	if(element instanceof IProject){
    		p = (IProject)element;
    		TreeDB db = TreesTracker.getTracker().getTree(p);
    		if(db!=null){
    			nodeToStore = db.getRootNode();
    			if(sbllist!=null)
    				sbllist.select(getSortByLocationsState()?1:0);
    		}
    	}
    	
    };
    //actions when new selection is performed
    @Override 
    public boolean performOk() {
        nodeToStore.putAttribute(new Attribute(nodeToStore, AttributeType.BOOL,TreeNode.SORT_BY_LOCATIONS,sbllist.getSelectionIndex()==1));
        nodeToStore.saveAttributes();
        return true;
    }
    private boolean getSortByLocationsState(){
		if(nodeToStore == null)
			return true;
		Attribute a = nodeToStore.getAttribute(TreeNode.SORT_BY_LOCATIONS);
		return a == null || a.getValue().equals("true") || a.getValue().equals(true);
	}
	//Changing settings to default
    @Override 
    protected void performDefaults() {
        IPreferenceStore pref = Activator.getDefault().getPreferenceStore();
        pref.setValue(TreeNode.SORT_BY_LOCATIONS, true);
        sbllist.select(1);
        helpLabel.setText(sbllist.getSelectionIndex()==1?LOCATIONS_TEXT:NAMES_TEXT);
        helpLabel.getParent().layout();
        super.performDefaults();
    }

	public static boolean getSortFlagState(TreeNode rootNode) {
		Attribute a = rootNode.getAttribute(TreeNode.SORT_BY_LOCATIONS);
		return a != null && (a.getValue().equals("true") || a.getValue().equals(true));
	}
}
