/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Iterator;
import java.util.LinkedHashMap;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.nodetypes.ReportFolder;
import com.unitesk.requality.report.Template;

public class ReportTemplatesPropertySection extends TreeNodePropertiesTab {
    // Widgets
    private List templates;
    private Text namepan;
    private Text pathpan;
    private Button deletetemplate;
    private LinkedHashMap<String, Template> tpls = new LinkedHashMap<String, Template>();
    private String selectedTemplateId;
    @Override
    protected void setTreeNode(TreeNode node) {
        if(!node.getType().equals(ReportFolder.TYPE_NAME)) {
            return;
        }
        
        selectedTemplateId = tpls.keySet().iterator().next();
        Iterator<String> it = tpls.keySet().iterator();
        String curId = "";
        int i = 0;
        for (; i < tpls.size() && !selectedTemplateId.equals(curId); i++) {
            curId = it.next();
        }
        templates.select(i - 1);
    }
    
    protected boolean copyDirectory(File sourceLocation , File targetLocation) throws IOException {
        if (sourceLocation.isDirectory()) {
            if (targetLocation.exists()) {
                return false;
            }else{
                targetLocation.mkdir();
            }
            String[] children = sourceLocation.list();
            for (int i=0; i<children.length; i++) {
                if(!copyDirectory(new File(sourceLocation, children[i]),
                        new File(targetLocation, children[i])))
                    return false;
            }
        } else {
            InputStream in = new FileInputStream(sourceLocation);
            OutputStream out = new FileOutputStream(targetLocation);
            byte[] buf = new byte[1024];
            int len;
            while ((len = in.read(buf)) > 0) {
                out.write(buf, 0, len);
            }
            in.close();
            out.close();
        }
        return true;
    }
    protected void removeFile(File f) throws IOException {
        if (f.isDirectory()) {
          for (File c : f.listFiles())
              removeFile(c);
        }
        if (!f.delete())
          throw new FileNotFoundException("Failed to delete file: " + f);
    }
    @Override
    public void createControls(final Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        parent.setLayout(new GridLayout(2, false));
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));
        TabbedPropertySheetWidgetFactory wf = aTabbedPropertySheetPage.getWidgetFactory();
        //label name
        wf.createLabel(parent, "Name:");
        //text name
        namepan = wf.createText(parent,"",SWT.NONE);
        namepan.setEditable(false);
        namepan.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        //label path
        wf.createLabel(parent, "Path:");
        //text path
        pathpan = wf.createText(parent, "", SWT.NONE);
        pathpan.setEditable(false);
        pathpan.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        //composite for buttons
        Composite pbuttons = wf.createComposite(parent);
        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 2;
        pbuttons.setLayoutData(gd);
        pbuttons.setLayout(new GridLayout(2, false));
        //button for template import
        Button addtemplate = wf.createButton(pbuttons, "Add new template", SWT.NONE);
        addtemplate.addListener(SWT.MouseDown, new Listener() {
            @Override
            public void handleEvent(Event event) {
                try {
                    //ask for folder that contains template
                    DirectoryDialog dd = new DirectoryDialog(parent.getShell());
                    dd.setMessage("Select folder which contains template files");
                    String path = dd.open();
                    if(path == null || path.equals(""))
                        return ;
                    File from = new File(path),
                            to = new File(Activator.getPath("templates") + File.separatorChar + from.getName());
                    if(from.getAbsolutePath().equals(to.getAbsolutePath()))
                        return ;
                    if(!new File(from+File.separator+"template.xml").exists()){
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Add template",
                                "Target folder is not a template folder or requality already contains added template!");
                        return ;
                    }
                    //copy folder to "templates" path from activator
                    try {
                        copyDirectory(from, to);
                    } catch (IOException e) {
                    }
                    String answ = updateTemplates(path);
                    if (answ == null){
                        Template tmpl = getTemplate();
                        if(tmpl != null){
                            namepan.setText(tmpl.getName());
                            pathpan.setText(to.getPath());
                        }else{
                            namepan.setText("");
                            pathpan.setText("");
                        }
                    }else{
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Add template",
                                answ);
                        removeFile(to);
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        });
        //button for template deletion(not for "read-only" templates)
        deletetemplate = wf.createButton(pbuttons, "Delete template", SWT.NONE);
        deletetemplate.setEnabled(false);
        deletetemplate.addListener(SWT.MouseDown, new Listener() {
            @Override
            public void handleEvent(Event event) {
                Template t = getTemplate();
                if(t == null)
                    return;
                if(t.getReadOnly()){
                    MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Add template",
                                "Cannot delete read-only template");
                    return;
                }
                try {
                    removeFile(new File(t.getReportPath()));
                    updateTemplates(t.getReportPath());
                } catch (IOException e) {
                    e.printStackTrace();
                }
                namepan.setText("");
                pathpan.setText("");
            }
        });
        createSelectTemplatePanel(parent).addListener(SWT.Modify, new Listener() {
            @Override
            public void handleEvent(Event event) {
            }
        });
        //c.add
    }

    public Template getTemplate() {
        return tpls.get(selectedTemplateId);
    }
    /**
     * Update templates list. If it is not possible or errors occurs, then return error messages
     * @param path path of last template added
     * @return error message otherwise null
     */
    private String updateTemplates(String path){
        LinkedHashMap<String, Template> tp2 = tpls; 
        int tcnt = 0;
        tpls = new LinkedHashMap<String, Template>(Activator.getDefault()
                .getRenewedReportTemplates());
        if(tpls.size() == 0)
            return null;
        templates.removeAll();
        for (Template tpl : tpls.values()) {
            templates.add(tpl.getName());
            if (tp2.containsKey(tpl.getId()))
                tcnt++;
        }
        if(tcnt == tp2.size() && tpls.values().size() == tcnt)
            return "Target folder is not a template folder or requality already contains added template!";
        Iterator <Template> i = tpls.values().iterator();
        Iterator <String> iid = tpls.keySet().iterator();
        while (i.hasNext()){
            String id = iid.next();
            Template t = i.next();
            if(t.getReportPath().equals(path))
                selectedTemplateId = id;
        }
        templates.setSelection(new String[]{selectedTemplateId});
        
        templates.redraw();
        templates.update();
        return null;
    }
    private Composite createSelectTemplatePanel(Composite parent) {
        Composite panel = new Composite(parent, SWT.NONE);
        panel.setLayout(new FillLayout());
        GridData gd = new GridData(GridData.FILL_BOTH);
        gd.horizontalSpan = 2;
        panel.setLayoutData(gd);
        templates = new List(panel, SWT.BORDER | SWT.READ_ONLY | SWT.HORIZONTAL );
        tpls = new LinkedHashMap<String, Template>(Activator.getDefault()
                .getReportTemplates());
        if (tpls.size() == 0) {
            return panel;
        }
        if (selectedTemplateId == null) {
            selectedTemplateId = Template.DEFAULT_ID;
        }

        for (Template tpl : tpls.values()) {
            templates.add(tpl.getName());
        }
        Iterator<String> it = tpls.keySet().iterator();
        String curId = "";
        int i = 0;
        for (; i < tpls.size() && !selectedTemplateId.equals(curId); i++) {
            curId = it.next();
        }
        templates.select(i-1);
        templates.addMouseListener(new MouseListener() {
            
            @Override
            public void mouseUp(MouseEvent e) {
                // TODO Auto-generated method stub
                
            }
            
            @Override
            public void mouseDown(MouseEvent e) {
                // TODO Auto-generated method stub
                Iterator<String> it = tpls.keySet().iterator();
                for (int i = 0; i < templates.getSelectionIndex() + 1; i++) {
                    selectedTemplateId = it.next();
                }
                Template t = getTemplate();
                if( t != null){
                    namepan.setText(t.getName() == null ? "" : t.getName());
                    pathpan.setText(t.getReportPath() == null ? "" : t.getReportPath());
                    deletetemplate.setEnabled(!t.getReadOnly());
                }else{
                    namepan.setText("");
                    pathpan.setText("");
                }
            }
            
            @Override
            public void mouseDoubleClick(MouseEvent e) {
                // TODO Auto-generated method stub

            }
        });
        return panel;
    }
}
