/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextInputListener;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.properties.tabbed.AbstractPropertySection;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.core.INodeChangeListener;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.transaction.ITransactionListener;
import com.unitesk.requality.core.transaction.TransactionStorage;
import com.unitesk.requality.eclipse.core.TreesTracker;

public class SourcePropertySection extends AbstractPropertySection {
    private TextViewer source;
    private FontRegistry fontRegistry;
    private TabbedPropertySheetPage page;

    public SourcePropertySection() {
    	super();
        fontRegistry = new FontRegistry(PlatformUI.getWorkbench().getDisplay());
        fontRegistry.put("source", new FontData[] { new FontData("Monospaced",
                12, SWT.NORMAL) });
    }

    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        page = aTabbedPropertySheetPage;
        parent.setLayout(new FillLayout());
        source = new TextViewer(parent, SWT.MULTI | SWT.READ_ONLY);
        source.getControl().setFont(fontRegistry.get("source"));
        source.addTextInputListener(new ITextInputListener() {
			@Override
			public void inputDocumentChanged(IDocument oldInput, IDocument newInput) {
				if(source.getControl().isDisposed())
					return;
		        source.getControl().pack();
		        source.getControl().redraw();
		        source.getControl().getParent().pack();
		        source.getControl().getParent().layout();
		        page.resizeScrolledComposite();
			}
			
			@Override
			public void inputDocumentAboutToBeChanged(IDocument oldInput,
					IDocument newInput) {
				// TODO Auto-generated method stub
				
			}
		});
        //source.getControl().setLayoutData(new Fi);
    }
    private boolean transupdate = false;
    INodeChangeListener ncl = new INodeChangeListener() {
		@Override
		public void deleted(TreeNode source) {
			if(target != null && source.getUUId().equals(target.getUUId())){
                if(page.getControl().isDisposed()) {
                    return;
                }
                if (page.getCurrentTab() != null) {
                	TreeNode par = target.getParent();
                	if(par == null || par.isDisposed())
                		par = par.getTreeDB().getRootNode();
                	setInput(getPart(), new StructuredSelection(par));
                }	
			}
		}
		
		@Override
		public void attributeChange(final TreeNode source, String what) {
			if(page.getControl().isDisposed()) {
                return;
            }
			if(source.getTreeDB().getStorage() instanceof TransactionStorage){
				transupdate = true;				
			}else{
				openNode(source);
			}
		}

		@Override
		public void moved(TreeNode source, TreeNode oldParent,
				TreeNode newParent) {
			// TODO Auto-generated method stub
			
		}
	};
	ITransactionListener itl = new 
			ITransactionListener() {
				
				@Override
				public void undo(TransactionStorage tStorage) {	
					if(transupdate){
						openNode(target);
						transupdate = false;
					}
				}
				
				@Override
				public void redo(TransactionStorage tStorage) {					
					if(transupdate){
						openNode(target);
						transupdate = false;
					}
				}
				
				@Override
				public void execute(TransactionStorage tStorage) {
					if(transupdate){
						openNode(target);
						transupdate = false;
					}
				}
				
				@Override
				public void _dispose() {
					// TODO Auto-generated method stub
					
				}
			};
    @Override
    public void setInput(IWorkbenchPart part, ISelection selection) {
        super.setInput(part, selection);

        TreeNode node = TreeNodePropertiesTab.toTreeNode(selection);
        if(node==null) {
            return;
        }
        openNode(node);
        
    }
    private TreeNode target;
    private void openNode(final TreeNode node) {
		SourcePropertySection.this.getPart().getSite().getShell().getDisplay().asyncExec(new Runnable() {
			@Override
			public void run() {
				openNodeSafe(TreeNodePropertiesTab.toRequirementOrNode(node));
			}
		});
	}

	private void openNodeSafe(TreeNode node) {
    	if(target != null){
    		target.removeGuiChangeListener(ncl);
    		target.getTreeDB().removeTransactionListener(itl);
    	}
    	if(node == null)
    		return;

        target = node;
        target.addGuiChangeListener(ncl);
        target.getTreeDB().addTransactionListener(itl);
    	IFile f = (IFile) TreesTracker.getResourceByNode(node);
    	if(f == null || !f.exists()){
    		source.setInput(new Document(""));
    		return;
    	}
        Writer writer = new StringWriter();
        char[] buffer = new char[1024];
        try {
            InputStream in = f.getContents();
            Reader reader = new BufferedReader(new InputStreamReader(in,
                    f.getCharset()));
            int n;
            while ((n = reader.read(buffer)) != -1) {
                writer.write(buffer, 0, n);
            }
            source.setInput(new Document(writer.toString()));
            in.close();
            
        } catch (Exception e) {
            e.printStackTrace();
        }
        //page.getControl().pack();
        //source.getControl().forceFocus();
	}
}
