/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.ITreeLogic.IdContainsWrongChars;
import com.unitesk.requality.core.ITreeLogic.IdEmpty;
import com.unitesk.requality.core.ITreeLogic.SameIdExistsInChildren;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TestPurpose;

public class TestMainPropertySection extends TreeNodePropertiesTab {
    private TestPurpose testPurpose;

    private Text testPurposeID; // Test Case Number
    private Combo status; // Status
    private Text author; // Author
    private AttributesPanel attribPane; // Attributes
    private Text textName;

    private ControlDecoration testPurposeIDDecorator;

    public TestMainPropertySection() {
    }

    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        TabbedPropertySheetWidgetFactory wf = getWidgetFactory();

        parent.addFocusListener(new FocusListener() {
            @Override
            public void focusLost(FocusEvent e) {
            }

            @Override
            public void focusGained(FocusEvent e) {
            }
        });
        parent.setLayout(new GridLayout(2, false));
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));

        wf.createLabel(parent, "Id:").setLayoutData(
                new GridData(GridData.HORIZONTAL_ALIGN_END));

        testPurposeID = wf.createText(parent, "", SWT.BORDER);
        testPurposeID.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        testPurposeIDDecorator = createDecorator(testPurposeID);
        testPurposeIDDecorator.hide();
        testPurposeID.addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
            	if(updating)
					return;
            	if(testPurpose == null)
                    return;
                String newId = testPurposeID.getText().trim();
                testPurposeID.setText(newId);
                if (testPurpose.getId().equals(newId)) {
                    return;
                }
                if (!testPurpose.getQualifiedId().equals(
                        Requirement.getTypeRootQId())) {
                    if (testPurpose.getTreeDB().getTreeLogic().isIdValid(testPurpose, newId) && testPurpose.getTreeDB().startTransaction(
                                "Update Test Purpose",true)) {
                        testPurpose.setId(newId);
                        testPurpose.getTreeDB().commit();
                    } else {
                        testPurposeID.setText(testPurpose.getId());
                    }
                }
            }
        });
        
        wf.createLabel(parent, "Name:").setLayoutData(
                new GridData(GridData.HORIZONTAL_ALIGN_END));
        
        textName = wf.createText(parent, "");
        textName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        textName.addFocusListener(new FocusAdapter() {
        	public void focusLost(FocusEvent e) {
        		if(updating)
    				return;
        		if (!textName.getText().equals(textName.getText().trim())) {
                    textName.setText(textName.getText().trim());
                }
                if (node == null) {
                    return;
                }
                if (node.getName() == null && textName.getText().equals("") || node.getName() != null && node.getName().equals(textName.getText())) {
                    return;
                }
               	if(node.getTreeDB().startTransaction("Update Requirement",true)){
               		node.setName(textName.getText());
               		node.saveAttributes();
               		node.getTreeDB().commit();
               	}else
               		textName.setText(node.getName()!=null?node.getName():"");
            }
        });

        wf.createLabel(parent, "Status:").setLayoutData(
                new GridData(GridData.HORIZONTAL_ALIGN_END));

        status = new Combo(parent, SWT.BORDER | SWT.READ_ONLY);
        status.setItems(new String[] { "in process", "complete", "verified" });
        status.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                if(testPurpose == null)
                    return;
                String oldStatus = (String) testPurpose
                        .getAttributeValue(TestPurpose.ATTR_STATUS);
                if (oldStatus == null) {
                    oldStatus = "";
                }
                String newStatus = "";
                newStatus = status.getText();
                if (oldStatus.equals(newStatus)) {
                    return;
                }
                if(testPurpose.getTreeDB().startTransaction("Update Test Purpose",true)){
                	testPurpose.putAttribute(new Attribute(testPurpose,AttributeType.STRING,
                        TestPurpose.ATTR_STATUS, newStatus));
                	testPurpose.saveAttributes();
                	testPurpose.getTreeDB().commit();
                }
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }
        });

        wf.createLabel(parent, "Author:").setLayoutData(
                new GridData(GridData.HORIZONTAL_ALIGN_END));

        author = wf.createText(parent, "", SWT.BORDER);
        author.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        author.addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
            	if(updating)
            		return;
                if(testPurpose == null)
                    return;
                String oldAuthor = (String) testPurpose
                        .getAttributeValue(TestPurpose.ATTR_AUTHOR);
                if (oldAuthor == null) {
                    oldAuthor = "";
                }
                String newAuthor = "";
                newAuthor = author.getText();
                if (oldAuthor.equals(newAuthor)) {
                    return;
                }
                if(testPurpose.getTreeDB().startTransaction("Update Test Purpose",true)){
                	if (!newAuthor.equals("")) {
                		testPurpose.putAttribute(new Attribute(
                				testPurpose,
                				AttributeType.STRING, TestPurpose.ATTR_AUTHOR,
                				newAuthor));
                	} else {
                		testPurpose.removeAttribute(TestPurpose.ATTR_AUTHOR);
                	}
                	testPurpose.saveAttributes();
                	testPurpose.getTreeDB().commit();
                }else
                	author.setText(oldAuthor);
            }
        });

        attribPane = new AttributesPanel(parent, wf);
        GridData gridData = new GridData(GridData.FILL_BOTH);
        gridData.horizontalSpan = 2;
        attribPane.getControl().setLayoutData(gridData);
    }

    private ControlDecoration createDecorator(Text text) {
        ControlDecoration controlDecoration = new ControlDecoration(text,
                SWT.LEFT | SWT.TOP);
        FieldDecoration fieldDecoration = FieldDecorationRegistry.getDefault()
                .getFieldDecoration(FieldDecorationRegistry.DEC_ERROR);
        controlDecoration.setImage(fieldDecoration.getImage());
        return controlDecoration;
    }

    @Override
    protected void setTreeNode(TreeNode node) {
        if (!(node instanceof TestPurpose) || testPurposeID == null
                || testPurposeID.isDisposed()) {
            return;
        }
        testPurpose = (TestPurpose) node;
        testPurposeID.setText(testPurpose.getId());
        status.select(status.indexOf((String) testPurpose.getAttributeValue(
                TestPurpose.ATTR_STATUS, "")));
        author.setText((String) testPurpose.getAttributeValue(
                TestPurpose.ATTR_AUTHOR, ""));
        attribPane.setNode(testPurpose);
        String name = node.getName();
        if(name == null)
        	name = "";
        textName.setText(name);
        
    }
}
