package com.unitesk.requality.report;

import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.UUID;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.unitesk.requality.core.FSStorage;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.cvs.CVSManager;
import com.unitesk.requality.eclipse.cvs.FileSpaceTimePoint;
import com.unitesk.requality.nodetypes.NodeType;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TestPurpose;
import com.unitesk.requality.tests.CoverageStorage;

public class StatisticManager {
	private IProject project;
	private String path;
	private Requirement req;
	private CoverageStorage cs;
	private FileSpaceTimePoint svnInfo;
	private String error = null;
	
	private ArrayList<String> revisionList = new ArrayList<String>();
	private ArrayList<String> revEmptyList = new ArrayList<String>();
	private String speshRev = "";
	private Long dateFrom=0l, dateTo=0l;
	
	private Integer rq = 0;
	private Integer tp = 0;
	private Integer rqtp = 0;
	private Map<String, Integer> covCount = new HashMap<String, Integer>();
	private List<UUID> calculated = new ArrayList<UUID>();
	private Map<UUID, Boolean> cov = new HashMap<UUID, Boolean>();

	public StatisticManager(IProject project, String path) {
		this.project = project;
		this.path = path.substring(5, path.length() - 11);
		error = null;
	}

	public String getErrorMsg(){
		return error;
	}
	public boolean isRevExist(String numRev) {
		this.speshRev = "";
		try {
			File file = new File(path + File.separator + "statistics.xml");
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc = dBuilder.parse(file);
			doc.getDocumentElement().normalize();
			NodeList nlist = doc.getElementsByTagName("statistic");

			for (int s = nlist.getLength() - 1; s >= 0; s--) {
				Node node = nlist.item(s);
				if (node.getNodeType() == Node.ELEMENT_NODE) {
					Element elm = (Element) node;
					String attRev = elm.getAttribute("numRevision");
					if (numRev.equals(attRev)) {
						if (!node.hasChildNodes()
								&& !elm.hasAttribute("noChild")) {
							this.speshRev = numRev;
							break;
						}
					}
				}
			}
			return speshRev.equals("");
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (IOException e) {
			System.out.println("File doesn't exist.");
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}

	public void getRevRecord() {
		CVSManager cvsm = Activator.getDefault().getCVSManager();
		Set<FileSpaceTimePoint> ps = new HashSet<FileSpaceTimePoint>(
				cvsm.getRevisions(((IResource) this.project)));
			getPrevRevision(cvsm, ps);
	}

	public boolean isDateCorrect (String dateAfter, String dateBefore){
		SimpleDateFormat format = new SimpleDateFormat(
				"EEE MMM dd HH:mm:ss zzz yyyy", Locale.ENGLISH);
		try {
			this.dateFrom = format.parse(dateAfter).getTime();
			this.dateTo = format.parse(dateBefore).getTime();
			if (this.dateTo - this.dateFrom <= 0) {
				System.out
						.println("Wrong dates. Date 'from' can't be equals or after date 'to'.");
				return false;
			}
		} catch (ParseException e) {
			System.out.println("Wrong date format.");
			return false;
		}
		return true;
	}
	
	public void refreshRecordsList() {
		try{
			CVSManager cvsm = Activator.getDefault().getCVSManager();
			Set<FileSpaceTimePoint> ps = new HashSet<FileSpaceTimePoint>(
				cvsm.getRevisions(((IResource) this.project)));
			for (FileSpaceTimePoint pp : ps) {
				writeHeaderReport(pp);
			}
			if(cvsm.getLastRevError()!=null)
				error = cvsm.getLastRevError();
		}
		catch(Exception e){
			error = e.toString();
		}
	}

	public boolean isInfoExist() {
		this.revisionList = new ArrayList<String>();
		this.revEmptyList = new ArrayList<String>();
		this.speshRev = "";
		Boolean next = true;
		try {
			File file = new File(path + File.separator + "statistics.xml");
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc = dBuilder.parse(file);
			doc.getDocumentElement().normalize();
			NodeList nlist = doc.getElementsByTagName("statistic");
			
			SimpleDateFormat format = new SimpleDateFormat(
					"EEE MMM dd HH:mm:ss zzz yyyy", Locale.ENGLISH);
			int rev = -1;
			for (int s = nlist.getLength() - 1; s >= 0; s--) {
				Node node = nlist.item(s);
				if (node.getNodeType() == Node.ELEMENT_NODE) {
					Element elm = (Element) node;
					Long attTime = format.parse(elm.getAttribute("time"))
							.getTime();
					if (this.dateFrom == 0 && this.dateTo == 0) {
						if (!node.hasChildNodes()
								&& !elm.hasAttribute("noChild")) {
							revisionList.add(elm.getAttribute("numRevision"));
						}
					} else if ((this.dateFrom <= attTime)
							&& (this.dateTo >= attTime)) {
							rev = s;
						if (!node.hasChildNodes()
								&& !elm.hasAttribute("noChild")) {
							revisionList.add(elm.getAttribute("numRevision"));
						}
					} else if (this.dateFrom > attTime && next) {
						if (!node.hasChildNodes()
								&& !elm.hasAttribute("noChild")) {
							revEmptyList.add(elm.getAttribute("numRevision"));
						} else {
							next = false;
						}
					}
				}
			}
			rev--;
			if (rev >= 0) {
				Node nodePrev = nlist.item(rev);
				if (nodePrev.getNodeType() == Node.ELEMENT_NODE) {
					Element elmPrev = (Element) nodePrev;
					if (!nodePrev.hasChildNodes()) {
						speshRev = elmPrev.getAttribute("numRevision");
					}
				}
			}
			return (this.revisionList.isEmpty() && speshRev.equals(""));
		} catch (ParseException e) {
			System.out.println("Wrong date format.");
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (IOException e) {
			System.out.println("File doesn't exist.");
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}

	public void getDateRecord() {
		CVSManager cvsm = Activator.getDefault().getCVSManager();
		Set<FileSpaceTimePoint> ps = new HashSet<FileSpaceTimePoint>(
				cvsm.getRevisions(((IResource) this.project)));
		List<FileSpaceTimePoint> revList = getRevisions(cvsm, ps);
		for (FileSpaceTimePoint rev : revList) {
			getRevisionInfo(this.project, cvsm, rev);
		}
		if (!speshRev.equals("")) {
			getPrevRevision(cvsm, ps);
		}
	}

	public List<FileSpaceTimePoint> getRevisions(CVSManager cvsm, Set<FileSpaceTimePoint> ps) {
		Map<String, FileSpaceTimePoint> map = new HashMap<String, FileSpaceTimePoint>();
			for (FileSpaceTimePoint pp : ps) {
			if (this.revisionList.contains(pp.getRevision()))
				map.put(pp.getRevision(), pp);
			}
		SortedSet<Integer> sortedRev = new TreeSet<Integer>();
		Map<Integer, String> ii = new HashMap<Integer, String>();
		for (String s : map.keySet()) {
			ii.put(Integer.parseInt(s), s);
			sortedRev.add(Integer.parseInt(s));// TODO
		}
		List<FileSpaceTimePoint> l = new LinkedList<FileSpaceTimePoint>();
		for (Integer i : sortedRev)
			l.add(map.get(ii.get(i)));
		return l;
	}

	public void getRevisionInfo(IProject proj, CVSManager cvsm, FileSpaceTimePoint pp) {
		try {
			TreeDB odb = TreesTracker.getTracker().getTree((IProject) proj);

			java.io.File f = cvsm
					.getCVSClient(((IResource) proj).getProject())
					.getContainerFromRepoS(pp.getPath(), pp.getRevision(), null);
			FSStorage fs = new FSStorage(f.getAbsolutePath() + "/");

			TreeDB db = TreeDB.load(fs);
			for (NodeType nt : odb.getTypes().values()) {
				db.addNodeType(nt.getTypeClass());
				}
			CoverageStorage cs = new CoverageStorage(db);
			try {
				createNewRecord(
						(Requirement) db.getNode(Requirement.getTypeRootQId()),
						cs, pp);
			} catch (NullPointerException e) {
				writeNoContentReport(pp.getRevision());
						}
		} catch (Exception e) {
			e.printStackTrace();
					}
				}

	private void getPrevRevision(CVSManager cvsm, Set<FileSpaceTimePoint> ps) {
		FileSpaceTimePoint rev = null;
					for (FileSpaceTimePoint pp : ps) {
			if (pp.getRevision().equals(this.speshRev)) {
				rev = pp;
				break;
					}
				}
		Boolean next = getPrevRevisionInfo(this.project, rev);
		Integer revNum = Integer.parseInt(speshRev) - 1;
		
		while ((!next) && (revNum > 0)) {
			speshRev = revNum.toString();
			if (revEmptyList.contains(speshRev)) {
				for (FileSpaceTimePoint pp : ps) {
					if (pp.getRevision().equals(this.speshRev)) {
						rev = pp;
						break;
			}
		}
				next = getPrevRevisionInfo(this.project, rev);
		}
			revNum--;
		}
	}
	private static TreeDB db;
	public boolean getPrevRevisionInfo(IProject proj, FileSpaceTimePoint pp) {
		try {
			CVSManager cvsm = Activator.getDefault().getCVSManager();
			TreeDB odb = TreesTracker.getTracker().getTree((IProject) proj);

			// if(odb == null)
			// continue;
			// получение всей полной инфы
			java.io.File f = cvsm
					.getCVSClient(((IResource) proj).getProject())
					.getContainerFromRepoS(pp.getPath(), pp.getRevision(), null);
			FSStorage fs = new FSStorage(f.getAbsolutePath() + "/");

			db = TreeDB.load(fs);
			for (NodeType nt : odb.getTypes().values()) {
				db.addNodeType(nt.getTypeClass());
			}
			CoverageStorage cs = new CoverageStorage(db);
			try {
				createNewRecord(
						(Requirement) db.getNode(Requirement.getTypeRootQId()),
						cs, pp);
				return true;
			} catch (NullPointerException e) {
				writeNoContentReport(pp.getRevision());
				return false;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}

	private void createNewRecord(Requirement rootReq, CoverageStorage cs,
			FileSpaceTimePoint svnInfo) {
		cs.loadStorage();
		req = rootReq;
		this.cs = cs;
		this.svnInfo = svnInfo;
		rq = 0;
		tp = 0;
		rqtp = 0;
		covCount.put("green", 0);
		covCount.put("yellow", 0);
		covCount.put("red", 0);
		calculated = new ArrayList<UUID>();
		cov = new HashMap<UUID, Boolean>();
		
		int sc = req.sizeChildren();
		if (req.getType().equals(Requirement.TYPE_NAME))
			rq += 1;
		if (cs.isCovered(req.getQualifiedId()))
			covCount.put("green", covCount.get("green") + 1);
		else {
			boolean cov = false;
			for (int i = 0; i < sc; i++) {
				TreeNode node = req.getChild(i);
				if (coveredChild(node) || cs.isCovered(node.getQualifiedId()))
					cov = true;
			}
			if (cov) {
				covCount.put("yellow", covCount.get("yellow") + 1);
			} else {
				covCount.put("red", covCount.get("red") + 1);
			}
		}
		for (int i = 0; i < sc; i++) {
			TreeNode node = req.getChild(i);
			this.calculateStatistics(node);
		}
		writeContentReport();
	}

	private void calculateStatistics(TreeNode sc) {
		if (calculated.contains(sc.getUUId()))
			return;
		// check node is requirement
		if (sc.getType().equals(Requirement.TYPE_NAME))
			rq += 1;
		// check node is test purpose
		if (sc.getType().equals(TestPurpose.TYPE_NAME))
			tp += 1;
		// check if node is directly covered by test
		if (cs.isCovered(sc.getQualifiedId()))
			covCount.put("green", covCount.get("green") + 1);
		else {
			if (coveredChild(sc)) {
				covCount.put("yellow", covCount.get("yellow") + 1);
			} else {
				covCount.put("red", covCount.get("red") + 1);
			}
		}
		for (int i = 0; i < sc.sizeChildren(); i++) {
			if (sc.getChild(i).getType().equals(TestPurpose.TYPE_NAME)) {
				rqtp += 1;
			}
		}
		// get first test path that covers node
		// String test =
		// cs.getCoverageInfo(node.getQualifiedId()).getTestPaths()[0];
		// cs.getErrorsForPath(test);
		calculated.add(sc.getUUId());
		for (int i = 0; i < sc.sizeChildren(); i++) {
			TreeNode node = sc.getChild(i);
			this.calculateStatistics(node);
		}
	}

	private boolean coveredChild(TreeNode node) {
		if (cov.containsKey(node.getUUId())) {
			return cov.get(node.getUUId());
		}
		cov.put(node.getUUId(), false);
		for (int j = 0; j < node.sizeChildren(); j++) {
			if (!node.getChild(j).getUUId().equals(node)
					&& (coveredChild(node.getChild(j)) || cs.isCovered(node
							.getChild(j).getQualifiedId()))) {
				cov.put(node.getUUId(), true);
				return true;
			}
		}
		return false;
	}
	DocumentBuilderFactory dbFactory = DocumentBuilderFactory
			.newInstance();
	DocumentBuilder dBuilder =null;
	private void writeHeaderReport(FileSpaceTimePoint pp) {
		try {
			File file = new File(path + File.separator + "statistics.xml");
			if(dBuilder==null)
				dBuilder = dbFactory.newDocumentBuilder();
			Document doc;

			if (file.exists()) {
				doc = dBuilder.parse(file);
				doc.getDocumentElement().normalize();
				boolean insert = false;
				Integer rev = Integer.parseInt(pp.getRevision());
				NodeList statList = doc.getElementsByTagName("statistic");
				for (int s = 0; s < statList.getLength(); s++) {
					Node statNode = statList.item(s);
					if (statNode.getNodeType() == Node.ELEMENT_NODE) {
						Element elm = (Element) statNode;
						Integer attRev = Integer.parseInt(elm
								.getAttribute("numRevision"));
						if (rev == attRev) {
								insert = true;
								break;
							}
						if (rev < attRev) {
								Element newChild = createHeaderElement(doc, pp);
								elm.getParentNode().insertBefore(newChild, elm);
								insert = true;
								break;
							}
						}
					}
				if (!insert) {
					Node stat = doc.getElementsByTagName("statistics").item(0);
					Element newChild = createHeaderElement(doc, pp);
					stat.appendChild(newChild);
				}
			} else {
				doc = dBuilder.newDocument();
				Element rootlvl1 = doc.createElement("statistics");
				doc.appendChild(rootlvl1);
				Element rootlvl2 = createHeaderElement(doc, pp);
				rootlvl1.appendChild(rootlvl2);
				file.getParentFile().mkdirs();
				file.createNewFile();
			}
			TransformerFactory tfFactory = TransformerFactory.newInstance();
			Transformer transformer = tfFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
			StreamResult strRes = new StreamResult(file);
			transformer.transform(source, strRes);
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private Element createHeaderElement(Document doc, FileSpaceTimePoint pp) {
		Element newChild = doc.createElement("statistic");
		if (pp != null) {
			newChild.setAttribute("time",
					new Date(pp.getEpochTimeSec()).toString());
			newChild.setAttribute("author", pp.getAuthor());
			newChild.setAttribute("numRevision", pp.getRevision());
		} else {
			newChild.setAttribute("time", new Date().toString());
			newChild.setAttribute("author", System.getProperty("user.name"));
			newChild.setAttribute("numRevision", "0");
		}
		return newChild;
	}

	private void writeContentReport() {
		try {
			File file = new File(path + File.separator + "statistics.xml");
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc;

			if (file.exists()) {
				doc = dBuilder.parse(file);
				doc.getDocumentElement().normalize();
				String rev = svnInfo.getRevision();
				NodeList statList = doc.getElementsByTagName("statistic");
				for (int s = 0; s < statList.getLength(); s++) {
					Node statNode = statList.item(s);
					if (statNode.getNodeType() == Node.ELEMENT_NODE) {
						Element elm = (Element) statNode;
						if (rev.equals(elm.getAttribute("numRevision"))) {
								Element node = doc.createElement("reqCount");
							node.setAttribute("name", "number of requirements");
								node.setAttribute("value", rq.toString());
								elm.appendChild(node);

								node = doc.createElement("tpCount");
							node.setAttribute("name", "number of test purposes");
								node.setAttribute("value", tp.toString());
								elm.appendChild(node);

								node = doc.createElement("reqtpCount");
								node.setAttribute("name",
										"number of requirements covered with test purposes");
								node.setAttribute("value", rqtp.toString());
								elm.appendChild(node);

								node = doc.createElement("statuses");
								for (Map.Entry<String, Integer> entry : covCount
										.entrySet()) {
								Element nodelvl2 = doc.createElement("status");
								nodelvl2.setAttribute("name", entry.getKey());
								nodelvl2.setAttribute("value", entry.getValue()
										.toString());
									node.appendChild(nodelvl2);
								}
								elm.appendChild(node);
							}
					}
				}
				TransformerFactory tfFactory = TransformerFactory.newInstance();
				Transformer transformer = tfFactory.newTransformer();
				DOMSource source = new DOMSource(doc);
				StreamResult strRes = new StreamResult(file);
				transformer.transform(source, strRes);
			} else {
				System.out.println("File" + path + File.separator
						+ "statistics.xml" + "doesn't exist!");
			}
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void writeNoContentReport(String revNum) {
		try {
			File file = new File(path + File.separator + "statistics.xml");
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc;

			if (file.exists()) {
				doc = dBuilder.parse(file);
				doc.getDocumentElement().normalize();

				NodeList statList = doc.getElementsByTagName("statistic");
				for (int s = 0; s < statList.getLength(); s++) {
					Node statNode = statList.item(s);
					if (statNode.getNodeType() == Node.ELEMENT_NODE) {
						Element elm = (Element) statNode;
						if (revNum.equals(elm.getAttribute("numRevision"))) {
							elm.setAttribute("noChild", "true");
						}
					}
				}
				TransformerFactory tfFactory = TransformerFactory.newInstance();
				Transformer transformer = tfFactory.newTransformer();
				DOMSource source = new DOMSource(doc);
				StreamResult strRes = new StreamResult(file);
				transformer.transform(source, strRes);
			} else {
				System.out.println("File" + path + File.separator
						+ "statistics.xml" + "doesn't exist!");
			}
		} catch (TransformerException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
