package com.unitesk.requality.report;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import javax.imageio.ImageIO;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class Statistics {
	private String path, pathGraph;
	private int width = 0;

	private int[] redPercent = new int[0];
	private int[] yellowPercent = new int[0];
	private int[] greenPercent = new int[0];
	private int[] redCount = new int[0];
	private int[] yellowCount = new int[0];
	private int[] greenCount = new int[0];

	private int[] rqCount = new int[0];
	private int[] tpCount = new int[0];
	private int[] rqtpCount = new int[0];
	private int[] rqPercent = new int[0];
	private int[] tpPercent = new int[0];
	private int[] rqtpPercent = new int[0];

	private String[] arrayDate = new String[0];
	private String[] arrayAuthor = new String[0];
	private String[] arrayRevision = new String[0];

	public Statistics() {
	}

	public Statistics(String path, String pathGraph) {
		this.path = path.substring(5, path.length() - 11);
		if (pathGraph.startsWith("file:"))
			pathGraph = pathGraph.substring(5);
		this.pathGraph = pathGraph;
	}

	public Map<String, Integer> getArrayData(String numRev) {
		Map<String, Integer> data = new HashMap<String, Integer>();
		Integer num = -1;
		for (int i = 0; i < arrayRevision.length; i++) {
			if (numRev.equals(arrayRevision[i])) {
				num = i;
			}
		}
		if (num >= 0) {
			data.put("number of requirements", rqCount[num] / 10);
			data.put("number of test purposes", tpCount[num] / 10);
			data.put("number of requirements covered with test purposes",
					rqtpCount[num] / 10);
			data.put("percent of requirements", rqPercent[num]/10);
			data.put("percent of test purposes", tpPercent[num]/10);
			data.put("percent of requirements covered with test purposes",
					rqtpPercent[num]/10);

			data.put("number coverage (green status)", greenCount[num] / 10);
			data.put("number coverage (yellow status)", yellowCount[num] / 10);
			data.put("number coverage (red status)", redCount[num] / 10);
			data.put("percent coverage (green status)", greenPercent[num] / 10);
			data.put("percent coverage (yellow status)",
					yellowPercent[num] / 10);
			data.put("percent coverage (red status)", redPercent[num] / 10);
			return data;
		} else {
			data.put("Error. Wrong revision number: " + numRev, 0);
			return data;
		}
	}

	public String[] getarrayDate() {
		return arrayDate;
	}

	public String[] getarrayRevision() {
		return arrayRevision;
	}

	public void readReport() {
		try {
			File file = new File(path + File.separator + "statistics.xml");
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc = dBuilder.parse(file);
			doc.getDocumentElement().normalize();
			NodeList nlist = doc.getElementsByTagName("statistic");
			width = 0;
			for (int s = 0; s < nlist.getLength(); s++) {
				Node node = nlist.item(s);
				if (node.getNodeType() == Node.ELEMENT_NODE) {
					Element elm = (Element) node;
					if (!elm.hasAttribute("noChild")) {
						width++;
					}
				}
			}

			arrayDate = new String[width];
			arrayAuthor = new String[width];
			arrayRevision = new String[width];
			redPercent = new int[width];
			yellowPercent = new int[width];
			greenCount = new int[width];
			redCount = new int[width];
			yellowCount = new int[width];
			greenPercent = new int[width];
			rqCount = new int[width];
			tpCount = new int[width];
			rqtpCount = new int[width];
			rqPercent = new int[width];
			tpPercent = new int[width];
			rqtpPercent = new int[width];

			int n = 0;
			for (int s = 0; s < nlist.getLength(); s++) {
				int sum = 0;
				int green = 0;
				int yellow = 0;
				int red = 0;
				Node node = nlist.item(s);
				if (node.getNodeType() == Node.ELEMENT_NODE) {
					Element elm = (Element) node;
					if (!elm.hasAttribute("noChild")) {
						arrayDate[n] = elm.getAttribute("time");
						arrayAuthor[n] = elm.getAttribute("author");
						arrayRevision[n] = elm.getAttribute("numRevision");
						if (node.hasChildNodes()) {
							try {
								rqCount[n] = Integer.parseInt(elm
										.getElementsByTagName("reqCount")
										.item(0).getAttributes()
										.getNamedItem("value").getNodeValue()) * 10;
								tpCount[n] = Integer.parseInt(elm
										.getElementsByTagName("tpCount")
										.item(0).getAttributes()
										.getNamedItem("value").getNodeValue()) * 10;
								rqtpCount[n] = Integer.parseInt(elm
										.getElementsByTagName("reqtpCount")
										.item(0).getAttributes()
										.getNamedItem("value").getNodeValue()) * 10;
							} catch (NumberFormatException e) {
								System.err.println("String wrong format!");
							}

							NodeList nlelm = elm
									.getElementsByTagName("statuses");
							Element nelm = (Element) nlelm.item(0);
							NodeList selm = nelm.getElementsByTagName("status");
							for (int l = 0; l < selm.getLength(); l++) {
								Element celm = (Element) selm.item(l);
								NamedNodeMap att = celm.getAttributes();
								try {
									if (att.getNamedItem("name").getNodeValue()
											.equals("green")) {
										sum += Integer.parseInt(att
												.getNamedItem("value")
												.getNodeValue());
										green += Integer.parseInt(att
												.getNamedItem("value")
												.getNodeValue());
									}
									if (att.getNamedItem("name").getNodeValue()
											.equals("yellow")) {
										sum += Integer.parseInt(att
												.getNamedItem("value")
												.getNodeValue());
										yellow += Integer.parseInt(att
												.getNamedItem("value")
												.getNodeValue());
									}
									if (att.getNamedItem("name").getNodeValue()
											.equals("red")) {
										sum += Integer.parseInt(att
												.getNamedItem("value")
												.getNodeValue());
										red += Integer.parseInt(att
												.getNamedItem("value")
												.getNodeValue());
									}
								} catch (NumberFormatException e) {
									System.err
											.println("String wrong format! String: "
													+ att.getNamedItem("value")
															.getNodeValue());
								}
							}

							greenCount[n] = green * 10;
							yellowCount[n] = yellow * 10;
							redCount[n] = red * 10;
							if (sum > 0) {
								greenPercent[n] = green * 1000 / sum;
								yellowPercent[n] = yellow * 1000 / sum;
								redPercent[n] = red * 1000 / sum;
							} else {
								System.out
										.println("Sum of coverages is zero! There is no coverage!");
							}
						}
						n++;
					}
				}
			}
			if (width > 0) {
				int max = 0;
				for (int j = 0; j < width; j++) {
					if (max < rqCount[j]) {
						max = rqCount[j];
					}
				}
				if (max != 0) {
					for (int j = 0; j < width; j++) {
						rqPercent[j] = rqCount[j] * 1000 / max;
					}
				}
				max = 0;
				for (int j = 0; j < width; j++) {
					if (max < tpCount[j]) {
						max = tpCount[j];
					}
				}
				if (max != 0) {
					for (int j = 0; j < width; j++) {
						tpPercent[j] = tpCount[j] * 1000 / max;
					}
				}
				max = 0;
				for (int j = 0; j < width; j++) {
					if (max < rqtpCount[j]) {
						max = rqtpCount[j];
					}
				}
				if (max != 0) {
					for (int j = 0; j < width; j++) {
						rqtpPercent[j] = rqtpCount[j] * 1000 / max;
					}
				}
			}
		} catch (FileNotFoundException e) {
			System.out.println(e.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public String generateGraphicSpecial(String measure, String index,
			String dateAfter, String dateBefore, String period, String step,
			String scaleX, String scaleY, String autoscale)
			throws ParseException {
		double x = 1, y = 1;
		Integer periodStep = 0, count;
		int max = 10, type = 0;

		try {
			periodStep = Integer.parseInt(step);
		} catch (NumberFormatException e) {
			System.out.println("Unparseable step: " + e.getLocalizedMessage());
			return "Wrong peroid step. Step should be an integer number more then [0].";
		}

		SimpleDateFormat format = new SimpleDateFormat(
				"EEE MMM dd HH:mm:ss zzz yyyy", Locale.ENGLISH);
		Calendar date = Calendar.getInstance(), dateFrom = Calendar
				.getInstance(), dateTo = Calendar.getInstance();
		try {
			dateFrom.setTime(format.parse(dateAfter));
			dateTo.setTime(format.parse(dateBefore));
			if (dateFrom.equals(dateTo) || dateFrom.after(dateTo)) {
				return "Wrong dates. Date 'from' can't be equals or after date 'to'.";
			}
		} catch (ParseException e) {
			System.out.println("Wrong date format: " + e.getLocalizedMessage());
			return "Wrong date format.";
		}

		ArrayList<Integer> arrayPoints = new ArrayList<Integer>();
		ArrayList<Date> Dates = new ArrayList<Date>();
		int width = 0, start = 0, to = -1;
		if (periodStep > 0) {
			if (dateFrom != null && dateTo != null) {
				int points = 0, from = 0;
				for (int i = 0; i < arrayDate.length; i++) {
					try {
						date.setTime(format.parse(arrayDate[i]));
						if (date.before(dateFrom)) {
							from = i + 1;
						} else if (date.equals(dateFrom)) {
							from = i;
						}
						if (date.before(dateTo) || date.equals(dateTo)) {
							to = i;
						}
					} catch (ParseException e) {
						System.out.println("Unparseable date: " + arrayDate[i]);
					} catch (NullPointerException e) {
						System.out.println("i = " + i);
						System.out.println("arrayDate.length = "
								+ arrayDate.length);
						System.out.println("arrayDate = " + arrayDate);
						e.printStackTrace();
					}
				}
				if ((to > 0) && (from < to)) {
					start = from;
					int prd = 0;
					if (period.equals("year")) {
						prd = Calendar.YEAR;
					} else if (period.equals("month")) {
						prd = Calendar.MONTH;
					} else if (period.equals("week")) {
						prd = Calendar.DATE;
						periodStep = periodStep * 7;
					} else if (period.equals("day")) {
						prd = Calendar.DATE;
					} else {
						return "Wrong peroid. Period should be: {day, week, month, year}.";
					}
					arrayPoints.add(from);
					while (dateFrom.before(dateTo)) {
						Dates.add((Date) dateFrom.getTime().clone());
						dateFrom.add(prd, periodStep);
						for (int i = from; i <= to; i++) {
							try {
								date.setTime(format.parse(arrayDate[i]));
								if (date.before(dateFrom)
										|| date.equals(dateFrom)) {
									points++;
									from = i + 1;
								}
							} catch (ParseException e) {
								System.out.println("Unparseable date: "
										+ arrayDate[i]);
							}
						}
						arrayPoints.add(points);
						points = 0;
					}
					Dates.add((Date) dateTo.getTime().clone());
					if ((arrayPoints.get(1) == 0) && (arrayPoints.get(0) > 0)) {
						arrayPoints.set(0, arrayPoints.get(0) - 1);
						arrayPoints.set(1, 1);
					}
					width = arrayPoints.size() - 1;
				} else {
					return "Too small date interval. There is only one revision between these dates.";
				}
			}
		}

		if (measure.equals("percent")) {
			max = 1000;
		} else if (measure.equals("count")) {
			if (index.equals("coverage")) {
				type = 1;
				for (int i = start; i <= to; i++) {
					if (max < greenCount[i]) {
						max = greenCount[i];
					}
				}
				for (int i = start; i <= to; i++) {
					if (max < yellowCount[i]) {
						max = yellowCount[i];
					}
				}
				for (int i = start; i <= to; i++) {
					if (max < redCount[i]) {
						max = redCount[i];
					}
				}
			} else if (index.equals("requirements")) {
				type = 2;
				for (int i = start; i <= to; i++) {
					if (max < rqCount[i]) {
						max = rqCount[i];
					}
				}
			} else if (index.equals("test purposes")) {
				type = 3;
				for (int i = start; i <= to; i++) {
					if (max < tpCount[i]) {
						max = tpCount[i];
					}
				}
			} else if (index.equals("requirements test purposes")) {
				type = 4;
				for (int i = start; i <= to; i++) {
					if (max < rqtpCount[i]) {
						max = rqtpCount[i];
					}
				}
			} else {
				return "Wrong index! Index should be {coverage, requirements, test purposes, requirements test purposes}.";
			}
		} else {
			return "Wrong measure! Measure should be percent or count.";
		}

		if (autoscale.equals("true")) {
			if (max * 2 * y + 50 != 450) {
				y = (450 - 50) / (double) (max * 2);
			}
			if (width * 20 * x + 75 != 500) {
				x = (500 - 75) / (double) (width * 20);
			}
		} else {
			try {
				if (scaleX != null) {
					x = Double.valueOf(scaleX) / 100;
				}
				if (width * 20 * x + 75 < 380) {
					x = (380 - 75) / (double) (width * 20);
					System.out.println("Too small scale! New scale of x is "
							+ x * 100 + "%");
				}
				if (width * 20 * x + 75 > 2000) {
					x = (2000 - 75) / (double) (width * 20);
					System.out.println("Too big scale! New scale of x is " + x
							* 100 + "%");
				}
				if (scaleY != null) {
					y = Double.valueOf(scaleY) / 100;
				}
				if (max * 2 * y + 50 < 450) {
					y = (450 - 50) / (double) (max * 2);
					System.out.println("Too small scale! New scale of y is "
							+ y * 100 + "%");
				}
				if (max * 2 * y + 50 > 2050) {
					y = (2050 - 50) / (double) (max * 2);
					System.out.println("Too big scale! New scale of y is " + y
							* 100 + "%");
				}
			} catch (NumberFormatException e) {
				System.out.println("Unparseable scale: "
						+ e.getLocalizedMessage());
				return "Wrong scale. Scale should be an integer number.";
			}
		}

		if (measure.equals("percent")) {
			if (y * 100 % 5 != 0) {
				y = (int) (y * 100 / 5) * 0.05;
				System.out
						.println("For percent measure y should be a multiple of five. New y is "
								+ y * 100 + "%");
			}
		}

		double n = 0;
		int ww = 1, hh = 1;

		BufferedImage ox = new BufferedImage(65, (int) (width * 20 * x) + 75,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D gOx = ox.createGraphics();
		gOx.setBackground(Color.white);
		gOx.clearRect(0, 0, ox.getWidth(), ox.getHeight());
		gOx.setColor(Color.black);
		SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy");
		SimpleDateFormat stf = new SimpleDateFormat("HH:mm:ss");
		gOx.setFont(new Font("TimesRoman", Font.PLAIN, 10));
		if (x < 1) {
			for (int i = 0; i <= width * 20 * x; i += 20) {
				gOx.drawString(sdf.format(Dates.get((int) n)), 1, i + 7);
				gOx.drawString(stf.format(Dates.get((int) n)), 1, i + 15);
				n = n + 1 / x;
				ww++;
			}
		} else {
			for (int i = 0; i <= width * 20 * x; i += 20 * x) {
				gOx.drawString(sdf.format(Dates.get((int) n)), 1, i + 7);
				gOx.drawString(stf.format(Dates.get((int) n)), 1, i + 15);
				n++;
				ww++;
			}
		}

		BufferedImage oy = new BufferedImage(50, (int) (max * 2 * y) + 50,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D gOy = oy.createGraphics();
		gOy.setBackground(Color.white);
		gOy.clearRect(0, 0, oy.getWidth(), oy.getHeight());
		gOy.setColor(Color.black);
		gOy.drawString(measure, 7, 10);
		if (y < 1) {
			for (int i = 0; i < max * y; i += 10) {
				count = (int) ((max - i / y) / 10);
				gOy.drawString(count.toString(), 10, i * 2 + 35);
				hh++;
			}
		} else {
			for (int i = 0; i < max; i += 10) {
				count = (max - i) / 10;
				gOy.drawString(count.toString(), 10, (int) (i * 2 * y) + 35);
				hh++;
			}
		}

		double imgX = 1, imgY = 1;
		if (x > 1) {
			imgX = x;
		}
		if (y > 1) {
			imgY = y;
		}
		BufferedImage img = new BufferedImage((int) (hh * 20 * imgY) + 25,
				(int) (ww * 20 * imgX), BufferedImage.TYPE_INT_RGB);
		Graphics2D g = img.createGraphics();
		g.setBackground(Color.white);
		g.clearRect(0, 0, img.getWidth(), img.getHeight());
		g.setColor(Color.black);

		if (measure.equals("percent")) {
			if (index.equals("coverage")) {
				paintSpecial(g, width, max,
						(ArrayList<Integer>) arrayPoints.clone(), greenPercent,
						x, y, 1, ww, hh);
				paintSpecial(g, width, max,
						(ArrayList<Integer>) arrayPoints.clone(),
						yellowPercent, x, y, 2, ww, hh);
				paintSpecial(g, width, max,
						(ArrayList<Integer>) arrayPoints.clone(), redPercent,
						x, y, 3, ww, hh);
			} else if (index.equals("requirements")) {
				paintSpecial(g, width, max, arrayPoints, rqPercent, x, y, 4,
						ww, hh);
			} else if (index.equals("test purposes")) {
				paintSpecial(g, width, max, arrayPoints, tpPercent, x, y, 4,
						ww, hh);
			} else if (index.equals("requirements test purposes")) {
				paintSpecial(g, width, max, arrayPoints, rqtpPercent, x, y, 4,
						ww, hh);
			}
		} else if (measure.equals("count")) {
			switch (type) {
			case 1:
				paintSpecial(g, width, max,
						(ArrayList<Integer>) arrayPoints.clone(), greenCount,
						x, y, 1, ww, hh);
				paintSpecial(g, width, max,
						(ArrayList<Integer>) arrayPoints.clone(), yellowCount,
						x, y, 2, ww, hh);
				paintSpecial(g, width, max,
						(ArrayList<Integer>) arrayPoints.clone(), redCount, x,
						y, 3, ww, hh);
				break;
			case 2:
				paintSpecial(g, width, max, arrayPoints, rqCount, x, y, 4, ww,
						hh);
				break;
			case 3:
				paintSpecial(g, width, max, arrayPoints, tpCount, x, y, 4, ww,
						hh);
				break;
			case 4:
				paintSpecial(g, width, max, arrayPoints, rqtpCount, x, y, 4,
						ww, hh);
				break;
			}
		}

		BufferedImage lg = new BufferedImage(345, 60,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D glg = lg.createGraphics();
		paintLegenda(glg, lg, index);

		AffineTransform at = AffineTransform.getRotateInstance(-Math.PI / 2, 0,
				0);
		BufferedImage nimg = new BufferedImage(img.getHeight() + 55,
				img.getWidth() + 150, BufferedImage.TYPE_INT_RGB);
		Graphics2D ng = nimg.createGraphics();
		ng.setBackground(Color.white);
		ng.clearRect(0, 0, nimg.getWidth(), nimg.getHeight());
		ng.drawImage(oy, -7, 20, null);
		ng.drawImage(lg, 10, img.getWidth() + 80, null);
		ng.transform(at);
		ng.drawImage(img, -img.getWidth() - 25, 40, null);
		ng.drawImage(ox, -img.getWidth() - 75, 50, null);

		ng.dispose();
		g.dispose();

		StringBuilder graphname = new StringBuilder(index + "_");
		graphname.append(measure + "_");
		graphname.append(period + "_");
		graphname.append(step + "_");
		graphname.append(dateFrom.getTimeInMillis() + "_");
		graphname.append(dateTo.getTimeInMillis());
		String imgPath = "<img style=\"max-width:900px; max-height:900px\" alt=\"User's Spesial Graphic\"><br>";
		try {
			File folder = new File(pathGraph + File.separator + "graphics");
			folder.mkdirs();
			ImageIO.write(nimg, "png", new File(pathGraph + File.separator
					+ "graphics" + File.separator + graphname + ".png"));
			imgPath = "<img style=\"max-width:900px; max-height:900px\" alt=\"User's Coverage Spesial Graphic\" src=\"./"
					+ "graphics/" + graphname + ".png\"><br>";
		} catch (IOException io) {
			io.printStackTrace();
		}
		return imgPath;
	}

	public void paintSpecial(Graphics2D g, int width, int height,
			ArrayList<Integer> arrayPoints, int[] array, double xper,
			double yper, int type, int ww, int hh) {
		double xw = 1, yh = 1;
		if (xper > 1) {
			xw = xper;
		}
		if (yper > 1) {
			yh = yper;
		}
		g.setStroke(new BasicStroke(1));
		g.setColor(Color.lightGray);
		for (int i = 0; i <= hh; i++) {
			g.drawLine((int) ((i * 10) * yh * 2 + 20), 0, (int) ((i * 10) * yh
					* 2 + 20), (int) (ww * xw * 20) + 21);
		}
		for (int i = 0; i <= ww; i++) {
			g.drawLine(15, (int) (i * 20 * xw + 15), (int) (hh * yh * 20) + 25,
					(int) (i * 20 * xw + 15));
		}
		g.setColor(Color.black);
		g.drawLine(20, 15, 20, (int) (ww * xw * 20) + 21);
		g.drawLine(20, 15, (int) (hh * yh * 20) + 25, 15);

		int[] y = { 21, 21 };
		int[] x = { 16, 16 };
		int from = (Integer) arrayPoints.get(0);

		int[] yArray = { 21 };
		int[] xArray = { 16 };
		if ((Integer) arrayPoints.get(1) != 0) {
			y[1] = (int) (array[from] * 2 * yper + 21);
		}
		g.setStroke(new BasicStroke(2));
		for (int i = 1; i <= width;) {
			if ((Integer) arrayPoints.get(i) != 0) {

				int length = (Integer) arrayPoints.get(i);
				yArray = new int[length * 4];
				xArray = new int[length * 4];

				int[] Array = new int[length];
				System.arraycopy(array, from, Array, 0, length);

				yArray[0] = y[1];
				for (int j = 0; j < Array.length; j++) {
					yArray[j * 4 + 1] = (int) (Array[j] * 2 * yper + 21);
					yArray[j * 4 + 2] = (int) (Array[j] * 2 * yper + 21);
					yArray[j * 4 + 3] = (int) (Array[j] * 2 * yper + 21);
					if ((j * 4 + 4) < yArray.length) {
						yArray[j * 4 + 4] = (int) (Array[j] * 2 * yper + 21);
					}
				}

				xArray[0] = x[1];
				int n = 0;
				for (int j = 1; j < xArray.length - 1; j++) {
					n++;
					xArray[j] = xArray[j - 1];
					j++;
					xArray[j] = xArray[j - 1];
					j++;
					xArray[j] = (int) (n * 20 * xper / length + xArray[0]);
					j++;
					if (j < xArray.length) {
						xArray[j] = xArray[j - 1];
					}
				}

				switch (type) {
				case 1:
					g.setColor(Color.green);
					break;
				case 2:
					g.setColor(new Color(255, 235, 0));
					break;
				case 3:
					g.setColor(Color.red);
					break;
				default:
					g.setColor(Color.blue);
				}
				g.drawPolyline(yArray, xArray, length * 4);

				from += (Integer) arrayPoints.get(i);
				arrayPoints.set(0, from);
				x[1] = xArray[xArray.length - 1];
				y[1] = yArray[yArray.length - 1];
				i++;
			} else {
				int n = 0;
				while ((Integer) arrayPoints.get(i) == 0) {
					i++;
					n++;
					if (i > width)
						break;
				}
				if (i <= width) {
					if (n > 1) {
						x[0] = xArray[xArray.length - 1];
						x[1] = (int) (20 * (i - 1) * xper + 16);
						y[0] = yArray[yArray.length - 1];
						y[1] = y[0];
						switch (type) {
						case 1:
							g.setColor(Color.green);
							break;
						case 2:
							g.setColor(new Color(255, 235, 0));
							break;
						case 3:
							g.setColor(Color.red);
							break;
						default:
							g.setColor(Color.blue);
						}
						g.drawPolyline(y, x, 2);

					} else {
						x[0] = xArray[xArray.length - 1];
						x[1] = (int) (20 * i * xper + 16);
						y[0] = yArray[yArray.length - 1];
						y[1] = y[0];

						switch (type) {
						case 1:
							g.setColor(Color.green);
							break;
						case 2:
							g.setColor(new Color(255, 235, 0));
							break;
						case 3:
							g.setColor(Color.red);
							break;
						default:
							g.setColor(Color.blue);
						}
						g.drawPolyline(y, x, 2);
					}
				} else {
					x[0] = xArray[xArray.length - 1];
					x[1] = (int) (20 * i * xper + 16);

					y[0] = y[1] = yArray[yArray.length - 1];
					switch (type) {
					case 1:
						g.setColor(Color.green);
						break;
					case 2:
						g.setColor(new Color(255, 235, 0));
						break;
					case 3:
						g.setColor(Color.red);
						break;
					default:
						g.setColor(Color.blue);
					}
					g.drawPolyline(y, x, 2);
				}
			}
		}
	}

	public String generateGraphic(String measure, String index, String scaleX,
			String scaleY, String autoscale) {
		double x = 1, y = 1;
		Integer count;
		int max = 10, type = 0;

		if (measure.equals("percent")) {
			max = 1000;
		} else if (measure.equals("count")) {
			if (index.equals("coverage")) {
				type = 1;
				for (int i = 0; i < greenCount.length; i++) {
					if (max < greenCount[i]) {
						max = greenCount[i];
					}
				}
				for (int i = 0; i < yellowCount.length; i++) {
					if (max < yellowCount[i]) {
						max = yellowCount[i];
					}
				}
				for (int i = 0; i < redCount.length; i++) {
					if (max < redCount[i]) {
						max = redCount[i];
					}
				}
			} else if (index.equals("requirements")) {
				type = 2;
				for (int i = 0; i < rqCount.length; i++) {
					if (max < rqCount[i]) {
						max = rqCount[i];
					}
				}
			} else if (index.equals("test purposes")) {
				type = 3;
				for (int i = 0; i < tpCount.length; i++) {
					if (max < tpCount[i]) {
						max = tpCount[i];
					}
				}
			} else if (index.equals("requirements test purposes")) {
				type = 4;
				for (int i = 0; i < rqtpCount.length; i++) {
					if (max < rqtpCount[i]) {
						max = rqtpCount[i];
					}
				}
			} else {
				return "Wrong index! Index should be {coverage, requirements, test purposes, requirements test purposes}.";
			}
		} else {
			return "Wrong measure! Measure should be percent or count.";
		}

		if (autoscale.equals("true")) {
			if (max * 2 * y + 50 != 450) {
				y = (450 - 50) / (double) (max * 2);
			}
			if (width * 20 * x + 75 != 500) {
				x = (500 - 75) / (double) (width * 20);
			}
		} else {
			try {
				if (scaleX != null) {
					x = Double.valueOf(scaleX) / 100;
				}
				if (width * 20 * x + 75 < 380) {
					x = (380 - 75) / (double) (width * 20);
					System.out.println("Too small scale! New scale of x is "
							+ x * 100 + "%");
				}
				if (width * 20 * x + 75 > 2000) {
					x = (2000 - 75) / (double) (width * 20);
					System.out.println("Too big scale! New scale of x is " + x
							* 100 + "%");
				}
				if (scaleY != null) {
					y = Double.valueOf(scaleY) / 100;
				}
				if (max * 2 * y + 50 < 450) {
					y = (450 - 50) / (double) (max * 2);
					System.out.println("Too small scale! New scale of y is "
							+ y * 100 + "%");
				}
				if (max * 2 * y + 50 > 2050) {
					y = (2050 - 50) / (double) (max * 2);
					System.out.println("Too big scale! New scale of y is " + y
							* 100 + "%");
				}
			} catch (NumberFormatException e) {
				System.out.println("Unparseable scale: "
						+ e.getLocalizedMessage());
				return "Wrong scale. Scale should be an integer number.";
			}
		}

		if (measure.equals("percent")) {
			if (y * 100 % 5 != 0) {
				y = (int) (y * 100 / 5) * 0.05;
				System.out
						.println("For percent measure y should be a multiple of five. New y is "
								+ y * 100 + "%");
			}
		}

		double n = 0;
		int ww = 1, hh = 1;

		BufferedImage ox = new BufferedImage((int) (width * 20 * x) + 75, 20,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D gOx = ox.createGraphics();
		gOx.setBackground(Color.white);
		gOx.clearRect(0, 0, ox.getWidth(), ox.getHeight());
		gOx.setColor(Color.black);
		if (x < 1) {
			for (int i = 0; i < width * 20 * x; i += 20) {
				gOx.drawString(arrayRevision[(int) n], i, 10);
				n = n + 1 / x;
				ww++;
			}
		} else {
			for (int i = 0; i < width; i++) {
				gOx.drawString(arrayRevision[i], (int) (i * 20 * x), 10);
				n++;
				ww++;
			}
		}

		BufferedImage oy = new BufferedImage(50, (int) (max * 2 * y) + 50,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D gOy = oy.createGraphics();
		gOy.setBackground(Color.white);
		gOy.clearRect(0, 0, oy.getWidth(), oy.getHeight());
		gOy.setColor(Color.black);
		gOy.drawString(measure, 7, 10);
		if (y < 1) {
			for (int i = 0; i < max * y; i += 10) {
				count = (int) ((max - i / y) / 10);
				gOy.drawString(count.toString(), 10, i * 2 + 35);
				hh++;
			}
		} else {
			for (int i = 0; i < max; i += 10) {
				count = (max - i) / 10;
				gOy.drawString(count.toString(), 10, (int) (i * 2 * y) + 35);
				hh++;
			}
		}
		double imgX = 1, imgY = 1;
		if (x > 1 && x != Double.POSITIVE_INFINITY) {
			imgX = x;
		}
		if (y > 1 && y != Double.POSITIVE_INFINITY) {
			imgY = y;
		}
		BufferedImage img = new BufferedImage((int) (hh * 20 * imgY) + 25,
				(int) (ww * 20 * imgX), BufferedImage.TYPE_INT_RGB);
		Graphics2D g2 = img.createGraphics();
		g2.setBackground(Color.white);
		g2.clearRect(0, 0, img.getWidth(), img.getHeight());
		if (measure.equals("percent")) {
			if (index.equals("coverage")) {
				paint(g2, width, max, greenPercent, x, y, 1, ww, hh);
				paint(g2, width, max, yellowPercent, x, y, 2, ww, hh);
				paint(g2, width, max, redPercent, x, y, 3, ww, hh);
			} else if (index.equals("requirements")) {
				paint(g2, width, max, rqPercent, x, y, 4, ww, hh);
			} else if (index.equals("test purposes")) {
				paint(g2, width, max, tpPercent, x, y, 4, ww, hh);
			} else if (index.equals("requirements test purposes")) {
				paint(g2, width, max, rqtpPercent, x, y, 4, ww, hh);
			}
		} else if (measure.equals("count")) {
			switch (type) {
			case 1:
				paint(g2, width, max, greenCount, x, y, 1, ww, hh);
				paint(g2, width, max, yellowCount, x, y, 2, ww, hh);
				paint(g2, width, max, redCount, x, y, 3, ww, hh);
				break;
			case 2:
				paint(g2, width, max, rqCount, x, y, 4, ww, hh);
				break;
			case 3:
				paint(g2, width, max, tpCount, x, y, 4, ww, hh);
				break;
			case 4:
				paint(g2, width, max, rqtpCount, x, y, 4, ww, hh);
				break;
			}
		}

		BufferedImage lg = new BufferedImage(345, 60,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D glg = lg.createGraphics();
		paintLegenda(glg, lg, index);

		AffineTransform at = AffineTransform.getRotateInstance(-Math.PI / 2, 0,
				0);
		BufferedImage nimg = new BufferedImage(img.getHeight() + 55,
				img.getWidth() + 130, BufferedImage.TYPE_INT_RGB);
		Graphics2D ng = nimg.createGraphics();
		ng.setBackground(Color.white);
		ng.clearRect(0, 0, nimg.getWidth(), nimg.getHeight());
		ng.drawImage(oy, -7, 20, null);
		ng.drawImage(lg, 10, img.getWidth() + 55, null);
		ng.drawImage(ox, 50, img.getWidth() + 25, null);
		ng.transform(at);
		ng.drawImage(img, -img.getWidth() - 25, 40, null);

		ng.dispose();
		g2.dispose();

		StringBuilder graphname = new StringBuilder("simple_");
		graphname.append(index + "_");
		graphname.append(measure);
		String imgPath = "<img style=\"max-width:900px; max-height:900px\" alt=\"Graphic\"><br>";
		try {
			File folder = new File(pathGraph + File.separator + "graphics");
			folder.mkdirs();
			ImageIO.write(nimg, "png", new File(pathGraph + File.separator
					+ "graphics" + File.separator + graphname + ".png"));
			imgPath = "<img style=\"max-width:900px; max-height:900px\" alt=\"Graphic\" src=\"./"
					+ "graphics/" + graphname + ".png\"><br>";
		} catch (IOException io) {
			io.printStackTrace();
		}
		return imgPath;
	}

	public void paint(Graphics2D g, int width, int height, int[] array,
			double x, double y, int type, int ww, int hh) {
		double xw = 1, yh = 1;
		if (x > 1 && x != Double.POSITIVE_INFINITY) {
			xw = x;
		}
		if (y > 1 && y != Double.POSITIVE_INFINITY) {
			yh = y;
		}
		g.setStroke(new BasicStroke(1));
		g.setColor(Color.lightGray);
		for (int i = 0; i <= hh; i++) {
			g.drawLine((int) ((i * 10) * yh * 2 + 20), 0, (int) ((i * 10) * yh
					* 2 + 20), (int) (ww * xw * 20) + 21);
		}
		for (int i = 0; i <= ww; i++) {
			g.drawLine(15, (int) (i * 20 * xw + 15), (int) (hh * yh * 20) + 25,
					(int) (i * 20 * xw + 15));
		}
		g.setColor(Color.black);
		g.drawLine(20, 15, 20, (int) (ww * xw * 20) + 21);
		g.drawLine(20, 15, (int) (hh * yh * 20) + 25, 15);

		int[] xArray = Arrays.copyOf(array, array.length);
		for (int j = 0; j < width; j++) {
			xArray[j] = (int) (xArray[j] * 2 * y + 21);
		}

		int[] yArray = new int[width];
		for (int i = 0; i < width; i++) {
			yArray[i] = (int) (i * 20 * x + 15);
		}

		switch (type) {
		case 1:
			g.setColor(Color.green);
			break;
		case 2:
			g.setColor(new Color(255, 235, 0));
			break;
		case 3:
			g.setColor(Color.red);
			break;
		default:
			g.setColor(Color.blue);
		}
		g.setStroke(new BasicStroke(2));
		g.drawPolyline(xArray, yArray, width);
	}

	private void paintLegenda(Graphics2D glg, BufferedImage lg, String index) {
		glg.setBackground(Color.white);
		glg.clearRect(0, 0, lg.getWidth(), lg.getHeight());
		glg.setColor(Color.black);
		glg.drawString("Legenda:", 1, 10);
		if (index.equals("coverage")) {
			glg.drawString("covered", 1, 25);
			glg.drawString("partly covered", 1, 40);
			glg.drawString("not covered", 1, 55);
			glg.setColor(Color.green);
			glg.drawLine(90, 20, 180, 20);
			glg.setColor(new Color(255, 235, 0));
			glg.drawLine(90, 35, 180, 35);
			glg.setColor(Color.red);
			glg.drawLine(90, 50, 180, 50);
		} else {
			glg.drawString(index, 1, 25);
			glg.setColor(Color.blue);
			glg.drawLine(1, 35, 91, 35);
		}
	}
}