/*
 * Copyright 2009-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.tools;

import java.util.Iterator;
import java.util.LinkedList;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.VirtualNode;

public class DeepFirstTreeWalker extends TreeWalker {
	boolean ignoreVirtual = false;
	public DeepFirstTreeWalker(TreeNode startNode,boolean ignoreVirtual) {
        super(startNode);
        this.ignoreVirtual = ignoreVirtual;
    }

    public DeepFirstTreeWalker(TreeNode startNode) {
        this(startNode,true);
    }

    @Override
    public Iterator<TreeNode> iterator() {
        return new Iterator<TreeNode>() {
            private LinkedList<TreeNode> stack = new LinkedList<TreeNode>();
            {
                stack.push(getStart());
            }

            @Override
            public boolean hasNext() {
                return !stack.isEmpty();
            }

            @Override
            public TreeNode next() {
                TreeNode result = stack.pop();
                while(result.isDisposed()&&stack.isEmpty())
                	result = stack.pop();
                if(!result.getType().equals(VirtualNode.TYPE_NAME)||!ignoreVirtual)
                	for (int i = result.sizeChildren() - 1; i >= 0; i--) {
                		stack.push(result.getChild(i));
                	}
                return result;
            }

            @Override
            public void remove() {
                throw new UnsupportedOperationException();
            }
        };
    }

}
